#!/usr/bin/env python
# -*- coding: utf-8 -*-

## Program:   VMTK
## Module:    $RCSfile: vmtkcenterlinesnetwork.py,v $
## Language:  Python
## Date:      $Date: 2018/04/24 09:52:56 $
## Version:   $Revision: 1.20 $

##   Copyright (c) Richard Izzo, Luca Antiga. All rights reserved.
##   See LICENCE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import vtkvmtk
from vmtk import pypes
from vmtk import vmtkcenterlines, vmtkcenterlinestonumpy, vmtknetworkextraction, vmtkdelaunayvoronoi, vmtknumpytocenterlines, vmtksurfacecapper
# joblib is optional: it is only used to parallelize the centerline
# computation, and execution falls back to serial mode when it is not
# installed (see UseJoblib below).
try:
    from joblib import Parallel, delayed
    JOBLIB_AVAILABLE = True
except ImportError:
    JOBLIB_AVAILABLE = False
import random
import numpy as np


def _compute_centerlines_network(surfaceAddress, delaunayAddress, voronoiAddress, poleIdsAddress, cell, points):
    '''a method to compute centerlines which can be called in parallel
    
    Arguments:
        surfaceAddress (str): the input memory address of the surface to calculate centerlines of
        delaunayAddress (str): the memory address of a previously computed delaunay triangulation of the 
            surface vtkUnstructuredGrid 
        voronoiAddress (str): the memory address of the previously computed voronoi diagram vtkPolyData
        poleIdsAddress (str): the memory address of the previously computed poleIds vtkIdList
        cell (np.array): the cellID connectivity list
        points (np.array): the x,y,z coordinates of points identified in the cell argument
    '''

    surface = vtk.vtkPolyData(surfaceAddress)
    delaunay = vtk.vtkUnstructuredGrid(delaunayAddress)
    voronoi = vtk.vtkPolyData(voronoiAddress)
    poleIds = vtk.vtkIdList(poleIdsAddress)

    cl = _compute_centerline_branch(surface, delaunay, voronoi, poleIds, cell, points)

    clConvert = vmtkcenterlinestonumpy.vmtkCenterlinesToNumpy()
    clConvert.Centerlines = cl
    clConvert.LogOn = 0
    clConvert.Execute()
    return clConvert.ArrayDict

def _compute_centerline_branch(surface, delaunay, voronoi, poleIds, cell, points):
    cellStartIdx = cell[0]
    cellEndIdx = cell[-1]
    cellStartPoint = points[cellStartIdx].tolist()
    cellEndPoint = points[cellEndIdx].tolist()
    cl = vmtkcenterlines.vmtkCenterlines()
    cl.Surface = surface
    cl.DelaunayTessellation = delaunay
    cl.VoronoiDiagram = voronoi
    cl.PoleIds = poleIds
    cl.SeedSelectorName = 'pointlist'
    # since we only set one target seed at a time, setting StopFastMarchingOnReachingTarget
    # greatly speeds up algorithm execution time.
    cl.StopFastMarchingOnReachingTarget = 1
    cl.SourcePoints = cellStartPoint
    cl.TargetPoints = cellEndPoint
    cl.LogOn = 0
    cl.Execute()
    return cl.Centerlines
    
class vmtkCenterlinesNetwork(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.Centerlines = None
        self.RadiusArrayName = 'MaximumInscribedSphereRadius'
        self.CostFunction = '1/R'

        self.EikonalSolutionArrayName = 'EikonalSolution'
        self.EdgeArrayName = 'EdgeArray'
        self.EdgePCoordArrayName = 'EdgePCoordArray'
        self.CostFunctionArrayName = 'CostFunctionArray'

        self.DelaunayTessellation = None
        self.VoronoiDiagram = None
        self.PoleIds = None
        self.RandomSeed = None
        
        
        # When using Joblib Under Windows, it is important to protect the main loop of code to avoid recursive spawning
        # of subprocesses. Since we cannot guarantee no code will run outside of “if __name__ == ‘__main__’” blocks
        # (only imports and definitions), we don't use joblib on windows.
        if (sys.platform == 'win32') or (sys.platform == 'win64') or (sys.platform == 'cygwin'):
            self.PrintLog('Centerlines extraction on windows computer will execute serially.')
            self.PrintLog('To speed up execution, please run vmtk on unix-like operating system and enable joblib')
            self.UseJoblib = False
        else:
            self.UseJoblib = True

        if self.UseJoblib and not JOBLIB_AVAILABLE:
            self.PrintLog('joblib is not installed; centerlines will be extracted serially.')
            self.PrintLog('To speed up execution, install the joblib package.')
            self.UseJoblib = False

        # The joblib parallelization hands data to the forked worker processes as
        # raw memory address strings, which requires the VTK Python wrappers to
        # support constructing an object from an address string. Modern VTK
        # removed that constructor form, so fall back to serial execution there.
        if self.UseJoblib:
            try:
                _addressProbe = vtk.vtkPolyData()
                vtk.vtkPolyData(_addressProbe.__this__)
            except TypeError:
                self.PrintLog('This VTK version does not support wrapping objects from memory addresses; centerlines will be extracted serially.')
                self.UseJoblib = False
        
        self.vmtkRenderer = None
        self.OwnRenderer = 0

        self.SetScriptName('vmtkcenterlinesnetwork')
        self.SetScriptDoc('compute centerlines from a branching tubular surface automatically.')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer']])
        self.SetOutputMembers([
            ['Centerlines','o','vtkPolyData',1,'','the output centerlines','vmtksurfacewriter'],
            ['RadiusArrayName','radiusarray','str',1,'','name of the array where radius values of maximal inscribed spheres are stored'],
            ['EikonalSolutionArrayName','eikonalsolutionarray','str',1],
            ['EdgeArrayName','edgearray','str',1],
            ['EdgePCoordArrayName','edgepcoordarray','str',1],
            ['CostFunction','costfunction','str',1,'','cost function minimized during centerline computation'],
            ['CostFunctionArrayName','costfunctionarray','str',1],
            ['DelaunayTessellation','delaunaytessellation','vtkUnstructuredGrid',1,'','','vmtkmeshwriter'],
            ['VoronoiDiagram','voronoidiagram','vtkPolyData',1,'','','vmtksurfacewriter'],
            ['PoleIds','poleids','vtkIdList',1],
            ['RandomSeed','randomseed','int',1],
            ['UseJoblib','usejoblib','bool',1]])

    def Execute(self):
        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        # feature edges are used to find any holes in the surface.
        fedges = vtk.vtkFeatureEdges()
        fedges.BoundaryEdgesOn()
        fedges.FeatureEdgesOff()
        fedges.ManifoldEdgesOff()
        fedges.SetInputData(self.Surface)
        fedges.Update()
        ofedges = fedges.GetOutput()

        # if numEdges is not 0, then there are holes which need to be capped
        numEdges = ofedges.GetNumberOfPoints()
        if numEdges != 0:
            tempcapper = vmtksurfacecapper.vmtkSurfaceCapper()
            tempcapper.Surface = self.Surface
            tempcapper.Interactive = 0
            tempcapper.Execute()

            networkSurface = tempcapper.Surface
        else:
            networkSurface = self.Surface

        # randomly select one cell to delete so that there is an opening for
        # vmtkNetworkExtraction to use.
        numCells = networkSurface.GetNumberOfCells()
        random_generator = random.Random()
        if self.RandomSeed is not None:
            random_generator.seed(self.RandomSeed)
        cellToDelete = random_generator.randrange(0, numCells-1)
        networkSurface.BuildLinks()
        networkSurface.DeleteCell(cellToDelete)
        networkSurface.RemoveDeletedCells()

        # extract the network of approximated centerlines
        net = vmtknetworkextraction.vmtkNetworkExtraction()
        net.Surface = networkSurface
        net.AdvancementRatio = 1.001
        net.Execute()
        network = net.Network

        convert = vmtkcenterlinestonumpy.vmtkCenterlinesToNumpy()
        convert.Centerlines = network
        convert.LogOn = False
        convert.Execute()
        ad = convert.ArrayDict
        cellDataTopology = ad['CellData']['Topology']

        # the network topology identifies an the input segment with the "0" id.
        # since we artificially created this segment, we don't want to use the
        # ends of the segment as source/target points of the centerline calculation
        nodeIndexToIgnore = np.where(cellDataTopology[:,0] == 0)[0][0]
        keepCellConnectivityList = []
        pointIdxToKeep = np.array([])
        removeCellLength = 0
        # we remove the cell, points, and point data which are associated with the
        # segment we want to ignore
        for loopIdx, cellConnectivityList in enumerate(ad['CellData']['CellPointIds']):
            if loopIdx == nodeIndexToIgnore:
                removeCellStartIdx = cellConnectivityList[0]
                removeCellEndIdx = cellConnectivityList[-1]
                removeCellLength = cellConnectivityList.size
                if (removeCellEndIdx + 1) - removeCellStartIdx != removeCellLength:
                    raise(ValueError)
                continue
            else:
                rescaledCellConnectivity = np.subtract(cellConnectivityList, removeCellLength, where=cellConnectivityList >= removeCellLength)
                keepCellConnectivityList.append(rescaledCellConnectivity)
                pointIdxToKeep = np.concatenate((pointIdxToKeep, cellConnectivityList)).astype(int)
        newPoints = ad['Points'][pointIdxToKeep]
        newRadius = ad['PointData']['Radius'][pointIdxToKeep]

        # precompute the delaunay tessellation for the whole surface.
        tessalation = vmtkdelaunayvoronoi.vmtkDelaunayVoronoi()
        tessalation.Surface = networkSurface
        tessalation.Execute()
        self.DelaunayTessellation = tessalation.DelaunayTessellation
        self.VoronoiDiagram = tessalation.VoronoiDiagram
        self.PoleIds = tessalation.PoleIds


        out = []
        self.PrintLog('Computing Centerlines ...')
        if self.UseJoblib and not JOBLIB_AVAILABLE:
            # UseJoblib may have been enabled from the command line even
            # though joblib is not installed.
            self.PrintLog('joblib is not installed; centerlines will be extracted serially.')
            self.UseJoblib = False
        if self.UseJoblib:
            # vtk objects cannot be serialized in python. Instead of converting the inputs to numpy arrays and having
            # to reconstruct the vtk object each time the loop executes (a slow process), we can just pass in the
            # memory address of the data objects as a string, and use the vtk python bindings to create a python name
            # referring to the data residing at that memory address. This works because joblib executes each loop
            # iteration in a fork of the original process, providing access to the original memory space.
            # However, the process does not work for return arguments, since the original process will not have access to
            # the memory space of the fork. To return results we use the vmtkCenterlinesToNumpy converter.
            networkSurfaceMemoryAddress = networkSurface.__this__
            delaunayMemoryAddress = tessalation.DelaunayTessellation.__this__
            voronoiMemoryAddress = tessalation.VoronoiDiagram.__this__
            poleIdsMemoryAddress = tessalation.PoleIds.__this__
            numParallelJobs = -1
            
            
            # note about the verbose function: while Joblib can print a progress bar output (set verbose = 20),
            # it does not implement a callback function as of version 0.11, so we cannot report progress to the user
            # if we are redirecting standard out with the self.PrintLog method.
            outlist = Parallel(n_jobs=numParallelJobs, backend='multiprocessing', verbose=0)(
                delayed(_compute_centerlines_network)(networkSurfaceMemoryAddress,
                                              delaunayMemoryAddress,
                                              voronoiMemoryAddress,
                                              poleIdsMemoryAddress,
                                              cell,
                                              newPoints) for cell in keepCellConnectivityList)
            for item in outlist:
                npConvert = vmtknumpytocenterlines.vmtkNumpyToCenterlines()
                npConvert.ArrayDict = item
                npConvert.LogOn = 0
                npConvert.Execute()
                out.append(npConvert.Centerlines)
        else:
            for cell in keepCellConnectivityList:
                    cl = _compute_centerline_branch(networkSurface, tessalation.DelaunayTessellation, tessalation.VoronoiDiagram,
                                                    tessalation.PoleIds, cell, newPoints)
                    out.append(cl)
                    

        # Append each segment's polydata into a single polydata object
        centerlineAppender = vtk.vtkAppendPolyData()
        for data in out:
            centerlineAppender.AddInputData(data)
        centerlineAppender.Update()

        # clean and strip the output centerlines so that redundant points are merged and tracts are combined
        centerlineCleaner = vtk.vtkCleanPolyData()
        centerlineCleaner.SetInputData(centerlineAppender.GetOutput())
        centerlineCleaner.Update()

        centerlineStripper = vtk.vtkStripper()
        centerlineStripper.SetInputData(centerlineCleaner.GetOutput())
        centerlineStripper.JoinContiguousSegmentsOn()
        centerlineStripper.Update()

        self.Centerlines = centerlineStripper.GetOutput()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      February 12, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkbranchgeometry as branchgeometry
from vtk.numpy_interface import dataset_adapter as dsa
import numpy as np


@pytest.fixture(scope='module')
def branch_geometry(aorta_centerline_branches):
    geometry = branchgeometry.vmtkBranchGeometry()
    geometry.Centerlines = aorta_centerline_branches
    geometry.BlankingArrayName = 'Blanking'
    geometry.RadiusArrayName = 'MaximumInscribedSphereRadius'
    geometry.GroupIdsArrayName = 'GroupIds'
    geometry.Execute()

    return geometry.GeometryData


@pytest.mark.parametrize("expectedKey",[
    ('Length'),
    ('Curvature'),
    ('Torsion'),
    ('Tortuosity'),
    ('GroupIds')
])
def test_expected_point_data_keys_present(branch_geometry, expectedKey):
    wrapped_branch_geom = dsa.WrapDataObject(branch_geometry)
    assert expectedKey in wrapped_branch_geom.PointData.keys()


def test_point_data_keys_count_is_5(branch_geometry):
    wrapped_branch_geom = dsa.WrapDataObject(branch_geometry)
    assert len(wrapped_branch_geom.PointData.keys()) == 5


def test_no_cell_data_keys_present(branch_geometry):
    wrapped_branch_geom = dsa.WrapDataObject(branch_geometry)
    assert len(wrapped_branch_geom.CellData.keys()) == 0


def test_branch_points_correct(branch_geometry):
    wrapped_branch_geom = dsa.WrapDataObject(branch_geometry)
    expectedPointLocations = np.array([[0, 0, 0],
                                       [0, 0, 0],
                                       [0, 0, 0]])
    assert np.allclose(wrapped_branch_geom.Points, expectedPointLocations) == True


@pytest.mark.parametrize("pointkey,expectedvalue",[
    ("Length", np.array([35.52495253829437, 35.604787458920114, 34.1129769167953])),
    ("Curvature", np.array([0.014414016946212475, 0.03419519613455192, 0.03416089030167796])),
    ("Torsion", np.array([-0.030372826702540885, -0.1759360023150921, -0.14364733046527386])),
    ("Tortuosity", np.array([0.01062993958363112, 0.01490173952950391, 0.01877139596625832])),
    ("GroupIds", np.array([0.0, 2.0, 3.0]))
])
def test_point_data_is_correct(branch_geometry, pointkey, expectedvalue):
    wrapped_branch_geom = dsa.WrapDataObject(branch_geometry)
    assert np.allclose(wrapped_branch_geom.PointData.GetArray(pointkey), expectedvalue) == True



#!/usr/bin/env python
# encoding: utf-8
"""
meshconvert.py

A script to convert between tetrahedral mesh formats.

Created by Bryan Klingner (stellar.b@overt.org) on 2006-12-07.
"""
from __future__ import print_function, absolute_import # NEED TO STAY AS TOP IMPORT
import sys
import getopt
import os

help_message = \
'''
meshconvert.py - a script to convert between tetrahedral mesh formats.

Usage:
   meshconvert.py [-s scale] input_file output_file
   
       input_file    the input tetrahedral (or trianglar surface) mesh file. 
                     This file must be in one of the following formats:
                         .node  - Jonathan Shewchuk's node file format.
                                  .node files must be accompanied by a
                                  .ele file that contains tetrahedra.
                         .mesh  - NETGEN's tetrahedral mesh format.
                         .vmesh - GRUMMP's tetrahedral mesh format.
                         .tet   - AIM@SHAPE repository's tetrahedral mesh format.
                         .off   - A common surface mesh format. If the input file
                                  is in .off format, only boundary triangles will
                                  be read and available for output.
                         .surf  - NETGEN's triangle surface mesh format. If the
                                  input file is in .surf format, only boundary
                                  triangles will be read and available for output.
                                  
       output_file   the output tetrahedral (or triangular surface) mesh file.
                     This file must be in one of the following formats:
                         .node  - Jonathan Shewchuk's mesh file format.
                                  An additional .ele file will also be 
                                  output that contains tetrahedra.
                         .mesh  - NETGEN's tetrahedral mesh format.
                         .obj   - A common surface mesh format. If the output
                                  file is in .obj format, only boundary triangles
                                  will be written to it.
                         .off   - A common surface mesh format. If the output
                                  file is in .off format, only boundary triangles
                                  will be written to it.
                         .surf  - NETGEN's tetrahedral mesh format. If the output
                                  file is in .surf format, only boundary triangles
                                  will be written to it.
       
       -s scale      optional vertex scale argument. All vertices geometric positions
                     will be multiplied by scale in the output file.
                     
       NOTE: As part of the conversion process, all tetrahedra will be adjusted to have
             consistent, right-handed orientation: That is, for a tetrahedron with vertices
             ordered (1, 2, 3, 4), the vertices 2, 3, 4 occur in counterclockwise order as 
             seen from vertex 1. If you curl the fingers of your right hand to follow the 
             vertices 2, 3, 4, then your thumb points toward vertex 1.
'''


class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg


def main(argv=None):
    # functions for reading
    readDict = {}
    readDict['.node'] = readNodeEle
    readDict['.mesh'] = readMesh
    readDict['.vmesh'] = readVmesh
    readDict['.tet'] = readTet
    readDict['.surf'] = readSurfTets
    readDict['.off'] = ReadOFFTets

    # functions for writing
    writeDict = {}
    writeDict['.node'] = writeNodeEle
    writeDict['.mesh'] = writeMesh
    writeDict['.obj'] = writeOBJTets
    writeDict['.off'] = writeOFFTets
    writeDict['.surf'] = writeSurfTets

    if argv is None:
        argv = sys.argv

    doscale = False

    # if they invoke with arguments, parse them
    if len(argv) > 1:
        try:
            try:
                opts, args = getopt.getopt(argv[1:], "hs:", ["help",])
            except getopt.error as msg:
                raise Usage(msg)

            # option processing
            for option, value in opts:
                if option == "-s":
                    doscale = True
                    scale = float(value)
                if option in ("-h", "--help"):
                    print(help_message)
                    return 1
        except Usage as err:
            print(sys.stderr, sys.argv[0].split("/")[-1] + ": " + str(err.msg))
            print(sys.stderr, "\t for help use --help")
            return 2
    else:
        print(help_message)
        return 1

    if len(argv) < 3:
        print("Not enough arguments. For help, use --help.")

    # determine the input and output formats, and check that they make sense
    inFileName = argv[-2]
    outFileName = argv[-1]
    inFileNameBase, inType = os.path.splitext(inFileName)
    outFileNameBase, outType = os.path.splitext(outFileName)

    if inType not in readDict.keys():
        print("Don't know how to read input format '%s'; invoke with --help for a list of supported formats." % (inType))
        return 2
    if outType not in writeDict.keys():
        print("Don't know how to write output format '%s'; invoke with --help for a list of supported formats." % (outType))
        return 2

    # read the input mesh
    points, tets, boundFaces = readDict[inType](inFileNameBase)
    if doscale:
        print("Scaling vertices by %g", scale)
        points = [vscale(scale, point) for point in points]
    # write the output mesh
    writeDict[outType](points, tets, boundFaces, outFileNameBase)


####
# Functions to READ tet mesh formats
####
def readTet(meshFileName):
    """Read .tet format... I don't actually remember who uses this"""

    # append .mesh to file stem
    meshFileName += '.tet'

    # open input .tet file
    infile = open(meshFileName)

    # fetch the number of points
    numpoints = int(infile.readline().split()[0])
    # fetch the number of tets
    numtets = int(infile.readline().split()[0])

    points = []
    # read in all the points
    for iter in range(0,numpoints):
        points.append(map(float,infile.readline().strip().split()))

    tets = []
    # read in the tets
    for iter in range(0,numtets):
        tets.append(vaddscalar(1,map(int,infile.readline().strip().split()[1:])))

    # correct orientation of tets
    for tetNum, tet in enumerate(tets):
        a = points[tet[0]-1]
        b = points[tet[1]-1]
        c = points[tet[2]-1]
        d = points[tet[3]-1]
        # if tet is negative orientation, flip two verts
        if orient3d(a,b,c,d) == 0:
            print("WHOA! input zero-volume tet...\n")
        if orient3d(a,b,c,d) < 0:
            temp = tet[0]
            tets[tetNum][0] = tet[1]
            tets[tetNum][1] = temp

    # this function doesn't attempt to recover boundary faces
    boundFaces = []

    infile.close()

    return points, tets, boundFaces


def readMesh(meshFileName):
    """Read .mesh files, as output by NETGEN"""

    # append .mesh to file stem
    meshFileName += '.mesh'

    # open input .mesh file
    infile = open(meshFileName)

    # fetch the number of points
    numpoints = int(infile.readline())

    points = []
    # read in all the points
    for iter in range(0,numpoints):
        points.append(map(float,infile.readline().strip().split()))

    # fetch the number of tets
    numtets = int(infile.readline())

    tets = []
    # read in the tets
    for iter in range(0,numtets):
        tets.append(map(int,infile.readline().strip().split()[1:]))

    # correct orientation of tets
    for tetNum, tet in enumerate(tets):
        a = points[tet[0]-1]
        b = points[tet[1]-1]
        c = points[tet[2]-1]
        d = points[tet[3]-1]
        # if tet is negative orientation, flip two verts
        if orient3d(a,b,c,d) == 0:
            print("WHOA! input zero-volume tet...\n")
            sys.exit(1)
        if orient3d(a,b,c,d) < 0:
            temp = tet[0]
            tets[tetNum][0] = tet[1]
            tets[tetNum][1] = temp

    # fetch the number of boundary faces
    numfaces = int(infile.readline())

    boundFaces = []
    # read in the boundary faces
    for iter in range(0,numfaces):
        boundFaces.append(map(int,infile.readline().strip().split()[1:]))

    infile.close()

    # fix tets to reference points starting at 0
    for index, tet in enumerate(tets):
        tets[index] = vsubscalar(1,tet)

    return points, tets, boundFaces


def readVmesh(meshFileName):
    """Read in .vmesh file... again, I can't recall who uses this format"""

    # append .vmesh to file stem
    meshFileName += '.vmesh'

    # open input .vmesh file
    infile = open(meshFileName)

    # first line: #tets #faces #boundFaces #verts
    firstLine = map(int,infile.readline().strip().split())
    numTets = firstLine[0]
    numFaces = firstLine[1]
    numBoundFaces = firstLine[2]
    numPoints = firstLine[3]

    points = []
    # read in all the points
    for iter in range(0,numPoints):
        points.append(map(float,infile.readline().strip().split()))

    faces = []
    face2tet = []
    for iter in range(0,numFaces):
        line = map(int,infile.readline().strip().split())
        face2tet.append(line[0:2])
        faces.append(line[2:])

    boundFaces = []
    for iter in range(0,numBoundFaces):
        boundFaces.append(map(int,infile.readline().strip().split()[2:]))
        boundFaces[-1][0] = boundFaces[-1][0] + 1
        boundFaces[-1][1] = boundFaces[-1][1] + 1
        boundFaces[-1][2] = boundFaces[-1][2] + 1
        boundFaces[-1].reverse()

    tets = [[]] * numTets
    mintet = 100
    maxtet = 0
    # reconstruct tets from face and face2tet information
    # for each face
    for faceNum, face in enumerate(faces):
        # for each vertex in the face
        for vert in face:
            # for each tet that has this face
            for tetNum in face2tet[faceNum]:
                # if this is a legit tet index
                if tetNum >= 0:
                    # use tetNum - 1 because .vmesh starts with tet 1
                    if (vert+1 in tets[tetNum]) == False:
                        tets[tetNum] = tets[tetNum] + [vert+1]

    # correct orientation of tets
    for tetNum, tet in enumerate(tets):
        a = points[tet[0]-1]
        b = points[tet[1]-1]
        c = points[tet[2]-1]
        d = points[tet[3]-1]
        # if tet is negative orientation, flip two verts
        if orient3d(a,b,c,d) == 0:
            print("WHOA! input zero-volume tet...\n")
        if orient3d(a,b,c,d) < 0:
            temp = tet[0]
            tets[tetNum][0] = tet[1]
            tets[tetNum][1] = temp

    return points, tets, boundFaces


def readNodeEle(filename, computeTopo=True):
    """Read a tetrahedral mesh in .node/.ele format, Jonathan Shewchuk's format.
       The .node file specifies the vertex locations and the .ele format specfies
       the tetrahedra. The .node file might start with an index of one or zero."""

    points, startFromZero = ReadNode(filename)
    tets = ReadEle(filename, startFromZero)

    # correct orientation of tets
    for tetNum, tet in enumerate(tets):
        a = points[tet[0]]
        b = points[tet[1]]
        c = points[tet[2]]
        d = points[tet[3]]
        # if tet is negative orientation, flip two verts
        if orient3d(a,b,c,d) == 0.0:
            print("WHOA! input zero-volume tet#%d...", tetNum)
            print("a=", ' '.join(['%0.18g' % x for x in a]))
            print("b=", ' '.join(['%0.18g' % x for x in b]))
            print("c=", ' '.join(['%0.18g' % x for x in c]))
            print("d=", ' '.join(['%0.18g' % x for x in d]))
        if orient3d(a,b,c,d) < 0.0:
            print("correcting inverted tet #%d", tetNum)
            temp = tet[0]
            tets[tetNum][0] = tet[1]
            tets[tetNum][1] = temp

    # build face topology information
    if computeTopo:
        faces, boundFaces, face2tet = GetFaceTopo(tets)
    else:
        boundFaces = None

    return points, tets, boundFaces


def GetFaceTopo(tets):
    """Recover topological information about faces"""

    # first, build list of all passible faces
    faces = []
    for tet in tets:
        # append each (outward oriented) face of the tet
        faces.append([tet[0], tet[1], tet[2]])
        faces.append([tet[0], tet[2], tet[3]])
        faces.append([tet[0], tet[3], tet[1]])
        faces.append([tet[1], tet[3], tet[2]])

    # sort the faces so the indices are in consistent order
    sortedFaces = [sorted(face) for face in faces]

    # get the unique faces
    uniqueFaces = unique(sortedFaces)

    uFaceDict = {}
    # build a dictionary of unique faces to speed lookup
    for facenum, face in enumerate(uniqueFaces):
        uFaceDict[tuple(face)] = facenum

    # build the tet -> face mapping by finding the index
    # into the unique face list for each face of each tet
    tet2face = []
    for tetNum in range(0, len(tets)):
        tet2face.append([uFaceDict[tuple(sortedFaces[tetNum*4])],
                        uFaceDict[tuple(sortedFaces[tetNum*4+1])],
                        uFaceDict[tuple(sortedFaces[tetNum*4+2])],
                        uFaceDict[tuple(sortedFaces[tetNum*4+3])]])

    # build the face -> tet mapping by finding the one or two tets
    # that contain each face in the unique face list
    face2tet = []
    for face in uniqueFaces:
        face2tet.append([-1, -1])

    for tetNum, tetfaces in enumerate(tet2face):
        for face in tetfaces:

            #print "creating face2tet, tetnum = %d, face = %d" % (tetNum, face)

            # if no tets have been recorded for this face, put it
            # in the first face
            if face2tet[face][0] == -1:
                face2tet[face][0] = tetNum
            else:
                # the second entry must not have a tet yet
                if (face2tet[face][1] != -1):
                    print("whoa, fount more than two tets for a face?")
                    print("tetnum is %d, face2tet[face] is" % tetNum, face2tet[face])
                assert(face2tet[face][1] == -1)
                face2tet[face][1] = tetNum

    #print "built face2tet."

    boundaryFaces = []
    # finally, get the list of boundary faces by building
    # list of all faces with just one tet
    for faceNum, facetets in enumerate(face2tet):
        if facetets[1] == -1:
            # this face has just one tet; it's a boundary face
            tet = tets[facetets[0]]
            tetfaces = [[tet[0], tet[1], tet[2]],
                        [tet[0], tet[2], tet[3]],
                        [tet[0], tet[3], tet[1]],
                        [tet[1], tet[3], tet[2]]]

            # find the properly oriented face
            foundface = False
            for face in tetfaces:
                if (uniqueFaces[faceNum] == sorted(face)):
                    #print "found correct oriented face [%d %d %d] corresponding to unique face [%d %d %d]" % (face[0], face[1], face[2], uniqueFaces[faceNum][0], uniqueFaces[faceNum][1], uniqueFaces[faceNum][2])
                    boundaryFaces.append(face)
                    foundface = True
            assert(foundface)
            foundface = False

    #print "there are %d boundary faces" % len(boundaryFaces)

    return uniqueFaces, boundaryFaces, face2tet


def readSurfTets(fileName):
    """A stub function to read surface faces from a .surf file,
       as output by NETGEN, as though it is a tet format"""
    points, tris = readSurf(fileName)

    return points, None, tris


def readSurf(fileName):
    """read .surf file format, containing the surface faces of a tet mesh
       as output by NETGEN"""
    inFileName = fileName + '.surf'

    # open input .surf file
    infile = open(inFileName)

    # read first line, make sure it's right
    header = infile.readline().strip()
    assert(header == 'surfacemesh')

    # read second line, number of vertices
    numPoints = int(infile.readline().strip())

    # read in all the points
    points = []

    for iter in range(0,numPoints):
        points.append(map(float,infile.readline().strip().split()))

    # read number of faces
    numTris = int(infile.readline().strip())

    # read in all the faces
    tris = []

    for iter in range(0,numTris):
        tri = map(int,infile.readline().strip().split())
        tri[0] = tri[0] - 1
        tri[1] = tri[1] - 1
        tri[2] = tri[2] - 1
        tris.append(tri)

    return points, tris


def ReadOFF(fileName):
    """Read OFF surface mesh format"""

    inFileName = fileName + '.off'

    # read input .off file
    infile = open(inFileName)

    # check for proper header
    header = infile.readline().strip()
    assert(header == 'OFF')

    # read second line "numverts numfaces 0"
    numPoints, numTris, blah = map(int, infile.readline().strip().split())

    print("reading OFF numPoints is %d numTris is %d" % (numPoints, numTris))

    # read in points
    points = []
    for iter in range(0,numPoints):
        points.append(map(float,infile.readline().strip().split()))

    # read in triangles
    tris = []

    for iter in range(0,numTris):
        tri = map(int,infile.readline().strip().split()[1:])
        tris.append(tri)

    return points, tris


def ReadOFFTets(fileName):
    """docstring for ReadOFFTet"""
    points, tris = ReadOFF(fileName)
    return points, None, tris

# read in a .node file (JRS' Pyramid format)


def ReadNode(fileName):
    inFileName = fileName + '.node'

    # open input .node file
    infile = open(inFileName)

    # fetch the number of points
    firstline = map(int,infile.readline().strip().split())
    numPoints = firstline[0]
    numMarkers = firstline[-1]

    # read in all the points
    points = []
    for iter in range(0,numPoints):
        # check whether point numbering starts at 1 (instead of 0)
        if iter == 0:
            firstline = infile.readline().strip().split()
            if firstline[0] == '1':
                startFromZero = False
            else:
                startFromZero = True
            # now put in the actual first point
            if numMarkers != 0:
                points.append(map(float,firstline[1:-numMarkers]))
            else:
                points.append(map(float,firstline[1:]))
        else:
            if numMarkers != 0:
                points.append(map(float,infile.readline().strip().split()[1:-numMarkers]))
            else:
                points.append(map(float,infile.readline().strip().split()[1:]))

    #print "in ReadNode startfrom zero is", startFromZero, "len(points) is", len(points)

    return points, startFromZero

# read tets from a .ele file (pyramid format)


def ReadEle(fileName, startFromZero=True):
    inFileName = fileName + '.ele'

    # open input .ele file
    infile = open(inFileName)

    # fetch the number of tets
    firstline = map(int,infile.readline().strip().split())
    numTets = firstline[0]
    numMarkers = firstline[-1]

    #print "in readele num markers is", numMarkers

    # read in all the tets
    tets = []
    for iter in range(0,numTets):
        # skip the tet number
        line = map(int,infile.readline().strip().split())
        if numMarkers == 0:
            tets.append(line[1:])
        else:
            tets.append(line[1:-numMarkers])

    # if vert indices started at one, decrement all the indices
    if startFromZero == False:
        for index, tet in enumerate(tets):
            tets[index] = vsubscalar(1,tet)

    #print "len(tets) is", len(tets)
    #maxpoint = max([max(tet) for tet in tets])
    #print "in readele max vertex index is", maxpoint

    return tets


def ReadTPoly(fileName):
    """read in tpoly output from adaptive skeleton climbing"""
    infile = open(fileName)

    meshString = infile.read()

    infile.close()

    triString = meshString[2:].split('\n3\n')

    vertCount = 0
    verts = []
    norms = []
    tris = []

    for tri in triString:
        trisplit = tri.split()
        verts.append(tuple(map(float,trisplit[0:3])))
        norms.append(tuple(map(float,trisplit[3:6])))
        verts.append(tuple(map(float,trisplit[6:9])))
        norms.append(tuple(map(float,trisplit[9:12])))
        verts.append(tuple(map(float,trisplit[12:15])))
        norms.append(tuple(map(float,trisplit[15:18])))
        tris.append((vertCount, vertCount+1, vertCount+2))
        vertCount += 3

    #print "verts:", verts
    #print "norms:", norms
    #print "tris:", tris

    return verts, tris


####
# Functions to WRITE meshes out to various formats
####
def writeMesh(points, tets, boundFaces, outFileName):
    outFileName += '.mesh'
    outfile = open(outFileName, 'w')

    # number of points
    outfile.write("%d\n" % len(points))
    # points
    for point in points:
        outfile.write(' ' + ' '.join(map(str,point)) + '\n')

    # number of tets
    outfile.write("%d\n" % len(tets))
    # tets
    for tet in tets:
        # add one because .mesh numbers verts from 1
        tet = [x + 1 for x in tet]
        outfile.write(" 1 " + ' '.join(map(str,tet)) + '\n')

    # number of boundary faces
    outfile.write("%d\n" % len(boundFaces))
    # boundary faces
    for face in boundFaces:
        # add one because .mesh numbers verts from 1
        face = [x + 1 for x in face]
        outfile.write(" 1 " + ' '.join(map(str,face)) + '\n')


def writeNode(points, outFileName):
    if outFileName.find('.node') == -1:
        outFileName += '.node'
    outfile = open(outFileName, 'w')

    # boilerplate first line
    outfile.write('%s 3 0 0\n' % len(points))

    # write out the points
    for iter in range(0,len(points)):
        outfile.write(str(iter+1) + ' ' + ' '.join(['%.18g' % x for x in points[iter]]) + '\n')
    pass

    outfile.close()


def writeEle(tets, outFileName):
    outFileName += '.ele'
    outfile = open(outFileName, 'w')

    # boilerplate first line
    outfile.write('%s 4 0 \n' % len(tets))

    for iter in range(0,len(tets)):
        tets[iter] = vaddscalar(1,tets[iter])
        outfile.write(str(iter+1) + ' ' + ' '.join(map(str,tets[iter])) + '\n')
    pass

    outfile.close()


def writeNodeEle(points, tets, boundFaces, outFileName):
    writeNode(points, outFileName)
    writeEle(tets, outFileName)


def writeOBJTets(points, tets, tris, outFileName):
    """docstring for writOBJTets"""
    writeOBJ(points, tris, outFileName)
    pass


def writeOBJ(verts, tris, outFileName):
    """write out an OBJ file from a list of vertices and triangles"""
    outFileName += '.obj'
    outfile = open(outFileName, 'w')

    for vert in verts:
        #print "vert:", vert
        outfile.write('v %.18g %.18g %.18g\n' % (vert[0], vert[1], vert[2]))
    for tri in tris:
        outfile.write('f %d %d %d\n' % (tri[0]+1, tri[1]+1, tri[2]+1))

    outfile.close()


def writeSurfTets(points, tets, tris, outFileName):
    """docstring for writOBJTets"""
    writeSurf(points, tris, outFileName)
    pass

# surf surface mesh format


def writeSurf(verts, tris, outFileName):
    """write out an surf file from a list of vertices and triangles"""
    outFileName += '.surf'
    outfile = open(outFileName, 'w')

    # write header lines
    outfile.write('surfacemesh\n')
    outfile.write('%d\n' % (len(verts)))

    for vert in verts:
        #print "vert:", vert
        outfile.write('%g %g %g\n' % (vert[0], vert[1], vert[2]))

    outfile.write('%d\n' % (len(tris)))

    for tri in tris:
        outfile.write('%d %d %d\n' % (tri[0], tri[1], tri[2]))

    outfile.close()


def writeOFFTets(points, tets, tris, outFileName):
    """docstring for writOBJTets"""
    writeOFF(points, tris, outFileName)
    pass

# OFF surface mesh format


def writeOFF(verts, tris, outFileName):
    """write out an OFF file from a list of vertices and triangles"""
    outFileName += '.off'
    outfile = open(outFileName, 'w')

    # write header lines
    outfile.write('OFF\n')
    outfile.write('%d %d 0\n' % (len(verts), len(tris)))

    for vert in verts:
        #print "vert:", vert
        outfile.write('%g %g %g\n' % (vert[0], vert[1], vert[2]))
    for tri in tris:
        outfile.write('3 %d %d %d\n' % (tri[0], tri[1], tri[2]))

    outfile.close()

## END file I/O functions


## Convenience math functions

# some simple geometry functions
def vadd(v1, v2):
    v3 = []
    # assumes vectors are of equal length
    for i in range(0,len(v1)):
        v3.append(v1[i] + v2[i])
    return v3


def vsub(v1, v2):
    v3 = []
    # assumes vectors are of equal length
    for i in range(0,len(v1)):
        v3.append(v1[i] - v2[i])
    return v3


def vlength(v):
    length = 0
    for ele in v:
        length += ele * ele
    return math.sqrt(length)


def vscale(scale, v):
    return [x * scale for x in v]


def vnorm(v):
    length = vlength(v)
    vscale(1/length,v)
    return v


def vaddscalar(scalar, v):
    return [x + scalar for x in v]


def vsubscalar(scalar, v):
    return [x - scalar for x in v]


def orient3d(a,b,c,d):
    """Compute the orientation of 4 points in 3D"""
    m11 = a[0] - d[0]
    m12 = a[1] - d[1]
    m13 = a[2] - d[2]
    m21 = b[0] - d[0]
    m22 = b[1] - d[1]
    m23 = b[2] - d[2]
    m31 = c[0] - d[0]
    m32 = c[1] - d[1]
    m33 = c[2] - d[2]

    det = (m11 * m22 * m33) + (m12 * m23 * m31) + (m13 * m21 * m32) - (m11 * m23 * m32) - (m12 * m21 * m33) - (m13 * m22 * m31)

    return det

# return all the unique items in a list


def unique(s):
    """Return a list of the elements in s, but without duplicates.

    For example, unique([1,2,3,1,2,3]) is some permutation of [1,2,3],
    unique("abcabc") some permutation of ["a", "b", "c"], and
    unique(([1, 2], [2, 3], [1, 2])) some permutation of
    [[2, 3], [1, 2]].

    For best speed, all sequence elements should be hashable.  Then
    unique() will usually work in linear time.

    If not possible, the sequence elements should enjoy a total
    ordering, and if list(s).sort() doesn't raise TypeError it's
    assumed that they do enjoy a total ordering.  Then unique() will
    usually work in O(N*log2(N)) time.

    If that's not possible either, the sequence elements must support
    equality-testing.  Then unique() will usually work in quadratic
    time.
    """

    n = len(s)
    if n == 0:
        return []

    # Try using a dict first, as that's the fastest and will usually
    # work.  If it doesn't work, it will usually fail quickly, so it
    # usually doesn't cost much to *try* it.  It requires that all the
    # sequence elements be hashable, and support equality comparison.
    u = {}
    try:
        for x in s:
            u[x] = 1
    except TypeError:
        del u  # move on to the next method
    else:
        return u.keys()

    # We can't hash all the elements.  Second fastest is to sort,
    # which brings the equal elements together; then duplicates are
    # easy to weed out in a single pass.
    # NOTE:  Python's list.sort() was designed to be efficient in the
    # presence of many duplicate elements.  This isn't true of all
    # sort functions in all languages or libraries, so this approach
    # is more effective in Python than it may be elsewhere.
    try:
        t = list(s)
        t.sort()
    except TypeError:
        del t  # move on to the next method
    else:
        assert n > 0
        last = t[0]
        lasti = i = 1
        while i < n:
            if t[i] != last:
                t[lasti] = last = t[i]
                lasti += 1
            i += 1
        return t[:lasti]

    # Brute force is all that's left.
    u = []
    for x in s:
        if x not in u:
            u.append(x)
    return u


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfaceresolution.py,v $
## Language:  Python
## Date:      $$
## Version:   $$

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this class was contributed by
##       Tangui Morvan
##       Kalkulo AS
##       Simula Research Laboratory

## This allows the user to specify a edge-length array to be used to specify resolution for surface remeshing
## The array is produced by RBF interpolation of values specified by the user by positioning spheres

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import vtkvmtk
from vmtk import vmtkrenderer
from vmtk import pypes


class vmtkSufaceResolution(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.ResolutionArrayName = 'ResolutionArray'
        self.RBFType = 'biharmonic'
        self.Spheres = vtk.vtkPolyData()
        self.vmtkRenderer = None
        self.OwnRenderer = 0
        self.DisplayArray = False
        self.SurfaceMapper = None
        self.CurrentSphereId = -1
        self.SphereWidget = None
        self.Opacity = 1.
        self.SpheresActor = None
        self.ScalarBarActor = None
        self.InteractionMode = 0
        self.ExamineSurface = None
        self.ExamineSpheres = vtk.vtkPolyData()
        self.ExamineSpheresActor = None
        self.ExamineText = None

        self.SetScriptName('vmtksurfaceresolution')
        self.SetScriptDoc('This allows the user to specify a edge-length array to be used to specify resolution for surface remeshing \
                           The array is produced by RBF interpolation of values specified by the user by positioning spheres')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['ResolutionArrayName','resolutionarray','str',1,'','array storing the desired edge length'],
            ['RBFType','rbftype','str',1,'["thinplatespline","biharmonic","triharmonic"]','the type of RBF interpolation'],
            ['Opacity','opacity','float',1,'(0.0,1.0)','object opacities in the scene'],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','','vmtksurfacewriter']
            ])

    def ComputeArray(self):
        rbf = vtkvmtk.vtkvmtkRBFInterpolation2()
        rbf.SetSource(self.Spheres)
        if self.RBFType == "thinplatespline":
            rbf.SetRBFTypeToThinPlateSpline()
        elif self.RBFType == "biharmonic":
            rbf.SetRBFTypeToBiharmonic()
        elif self.RBFType == "triharmonic":
            rbf.SetRBFTypeToTriharmonic()
        rbf.ComputeCoefficients()
        sampler = vtkvmtk.vtkvmtkPolyDataSampleFunction()
        sampler.SetInputData(self.Surface)
        sampler.SetImplicitFunction(rbf)
        sampler.SetSampleArrayName(self.ResolutionArrayName)
        sampler.Update()
        return sampler.GetOutput()

    def InitializeSpheres(self):
        if (self.InteractionMode==0):
            self.Spheres.Initialize()
            seedPoints = vtk.vtkPoints()
            self.Spheres.SetPoints(seedPoints)
            self.Spheres.GetPointData().Initialize()
            seedRadii = vtk.vtkDoubleArray()
            self.Spheres.GetPointData().SetScalars(seedRadii)
            self.CurrentSphereId = -1
            self.SphereWidget.Off()
        else:
            self.ExamineSpheres.Initialize()
            spherePoints = vtk.vtkPoints()
            self.ExamineSpheres.SetPoints(spherePoints)
            self.ExamineSpheres.GetPointData().Initialize()
            sphereRadii = vtk.vtkDoubleArray()
            self.ExamineSpheres.GetPointData().SetScalars(sphereRadii)

    def PlaceSphere(self):
        if self.CurrentSphereId == -1:
            return
        self.SphereWidget.SetCenter(self.Spheres.GetPoint(self.CurrentSphereId))
        self.SphereWidget.SetRadius(self.Spheres.GetPointData().GetScalars().GetValue(self.CurrentSphereId))

    def SphereCallback(self,widget,event_string):
        if self.CurrentSphereId == -1:
            return
        minRadius = self.Surface.GetLength()*0.001
        if self.SphereWidget.GetRadius() < minRadius:
            self.SphereWidget.SetRadius(minRadius)
        self.Spheres.GetPoints().SetPoint(self.CurrentSphereId,self.SphereWidget.GetCenter())
        self.Spheres.GetPointData().GetScalars().SetValue(self.CurrentSphereId,self.SphereWidget.GetRadius())
        self.Spheres.Modified()

    def UndoCallback(self,obj):
        self.InitializeSpheres()
        self.Spheres.Modified()
        self.vmtkRenderer.RenderWindow.Render()

    def PickCallback(self,obj):
        picker = vtk.vtkCellPicker()
        picker.SetTolerance(1E-4 * self.Surface.GetLength())
        eventPosition = self.vmtkRenderer.RenderWindowInteractor.GetEventPosition()
        #eventPosition = obj.GetEventPosition()
        result = picker.Pick(float(eventPosition[0]),float(eventPosition[1]),0.0,self.vmtkRenderer.Renderer)
        if result == 0:
            return
        pickPosition = picker.GetPickPosition()
        if (self.InteractionMode==0):
            self.CurrentSphereId = self.Spheres.GetPoints().InsertNextPoint(pickPosition)
            self.Spheres.GetPointData().GetScalars().InsertNextValue(self.Surface.GetLength()*0.01)
            self.Spheres.Modified()
            self.PlaceSphere()
            self.SphereWidget.On()
        else:
            pickedCellPointIds = self.Surface.GetCell(picker.GetCellId()).GetPointIds()
            minDistance = 1E10
            pickedPointId = -1
            for i in range(pickedCellPointIds.GetNumberOfIds()):
                distance = vtk.vtkMath.Distance2BetweenPoints(pickPosition,self.Surface.GetPoint(pickedCellPointIds.GetId(i)))
                if distance < minDistance:
                    minDistance = distance
                    pickedPointId = pickedCellPointIds.GetId(i)
            if pickedPointId == -1:
                pickedPointId = pickedCellPointIds.GetId(0)
            point = self.Surface.GetPoint(pickedPointId)
            self.ExamineSpheres.GetPoints().InsertNextPoint(point)
            length = 0.
            array = self.ExamineSurface.GetPointData().GetArray(self.ResolutionArrayName)
            if (array):
                length = array.GetComponent(pickedPointId,0)
            self.ExamineSpheres.GetPointData().GetScalars().InsertNextValue(length)
            self.ExamineSpheres.Modified()
        self.vmtkRenderer.RenderWindow.Render()

    def IncreaseSphereRadiusCallback(self,obj):
        if self.CurrentSphereId != -1:
            newval = self.Spheres.GetPointData().GetScalars().GetValue(self.CurrentSphereId) + self.Surface.GetLength()*0.01
            self.Spheres.GetPointData().GetScalars().SetValue(self.CurrentSphereId,newval)
            self.Spheres.Modified()
            self.PlaceSphere()
            self.vmtkRenderer.RenderWindow.Render()

    def DecreaseSphereRadiusCallback(self,obj):
        if self.CurrentSphereId != -1:
            newval = self.Spheres.GetPointData().GetScalars().GetValue(self.CurrentSphereId) - self.Surface.GetLength()*0.01
            if newval> 0:
                self.Spheres.GetPointData().GetScalars().SetValue(self.CurrentSphereId,newval)
                self.Spheres.Modified()
                self.PlaceSphere()
                self.vmtkRenderer.RenderWindow.Render()

    def NextCallback(self,obj):
        if self.CurrentSphereId != -1:
            self.CurrentSphereId = (self.CurrentSphereId + 1) % self.Spheres.GetNumberOfPoints();
            self.PlaceSphere()
            self.vmtkRenderer.RenderWindow.Render()

    def PreviousCallback(self,obj):
        if self.CurrentSphereId != -1:
            self.CurrentSphereId = (self.CurrentSphereId - 1) % self.Spheres.GetNumberOfPoints();
            self.PlaceSphere()
            self.vmtkRenderer.RenderWindow.Render()

    def DistancesCallback(self,obj):
        self.DisplayArray = not self.DisplayArray
        if self.DisplayArray:
            self.ExamineSurface = self.ComputeArray()
            self.SurfaceMapper.SetInputData(self.ExamineSurface)
            self.ExamineSurface.GetPointData().SetActiveScalars(self.ResolutionArrayName)
            array = self.ExamineSurface.GetPointData().GetScalars()
            if (array):
                array.Modified()
                self.SurfaceMapper.SetScalarRange(array.GetRange(0))
                self.ScalarBarActor.VisibilityOn()
        else:
            self.SurfaceMapper.SetInputData(self.Surface)
            self.ScalarBarActor.VisibilityOff()
        self.SurfaceMapper.SetScalarVisibility(self.DisplayArray)
        self.vmtkRenderer.RenderWindow.Render()

    def ExamineCallback(self,obj):
        #Switch beetween examien and interact mode
        if self.InteractionMode == 0:
            self.InteractionMode = 1
            self.ExamineSurface = self.ComputeArray()
            #self.SpheresActor.VisibilityOff()
            self.SphereWidget.Off()
            self.ExamineSpheresActor.VisibilityOn()
            self.ExamineText.VisibilityOn()
            self.InitializeSpheres()
        else:
            self.InteractionMode = 0
            #Compute the distances
            self.SpheresActor.VisibilityOn()
            self.ExamineSpheresActor.VisibilityOff()
            self.ExamineText.VisibilityOff()
            if (self.CurrentSphereId!=-1):
                self.SphereWidget.On()
        self.vmtkRenderer.RenderWindow.Render()

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        if not self.vmtkRenderer:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.vmtkRenderer.RegisterScript(self)

        glyphs = vtk.vtkGlyph3D()
        glyphSource = vtk.vtkSphereSource()
        glyphSource.SetRadius(1)
        glyphs.SetInputData(self.Spheres)
        glyphs.SetSourceConnection(glyphSource.GetOutputPort())
        glyphs.SetScaleModeToScaleByScalar()
        glyphs.SetScaleFactor(1.)
        glyphMapper = vtk.vtkPolyDataMapper()
        glyphMapper.SetInputConnection(glyphs.GetOutputPort())
        glyphMapper.ScalarVisibilityOff()
        self.SpheresActor = vtk.vtkActor()
        self.SpheresActor.SetMapper(glyphMapper)
        self.SpheresActor.GetProperty().SetColor(1.0,0.0,0.0)
        self.SpheresActor.GetProperty().SetOpacity(self.Opacity)
        self.SpheresActor.PickableOff()
        self.vmtkRenderer.Renderer.AddActor(self.SpheresActor)

        examineGlyphs = vtk.vtkGlyph3D()
        examineGlyphSource = vtk.vtkSphereSource()
        examineGlyphSource.SetRadius(1)
        examineGlyphs.SetInputData(self.ExamineSpheres)
        examineGlyphs.SetSourceConnection(examineGlyphSource.GetOutputPort())
        examineGlyphs.SetScaleModeToScaleByScalar()
        examineGlyphs.SetScaleFactor(1.)
        examineGlyphMapper = vtk.vtkPolyDataMapper()
        examineGlyphMapper.SetInputConnection(examineGlyphs.GetOutputPort())
        examineGlyphMapper.ScalarVisibilityOff()
        self.ExamineSpheresActor = vtk.vtkActor()
        self.ExamineSpheresActor.SetMapper(examineGlyphMapper)
        self.ExamineSpheresActor.GetProperty().SetColor(0.0,1.0,0.0)
        self.ExamineSpheresActor.GetProperty().SetOpacity(self.Opacity)
        self.ExamineSpheresActor.PickableOff()
        self.ExamineSpheresActor.VisibilityOff()
        self.vmtkRenderer.Renderer.AddActor(self.ExamineSpheresActor)

        self.vmtkRenderer.AddKeyBinding('u','Undo.',self.UndoCallback)
        self.vmtkRenderer.AddKeyBinding('space','Place picks.',self.PickCallback)
        self.vmtkRenderer.AddKeyBinding('+','Increase sphere radius.',self.IncreaseSphereRadiusCallback)
        self.vmtkRenderer.AddKeyBinding('-','Decrease sphere radius.',self.DecreaseSphereRadiusCallback)
        self.vmtkRenderer.AddKeyBinding('n','Skip to next sphere.',self.NextCallback)
        self.vmtkRenderer.AddKeyBinding('v','Skip to previous sphere.',self.PreviousCallback)
        self.vmtkRenderer.AddKeyBinding('d','Show distances graph.',self.DistancesCallback)
        self.vmtkRenderer.AddKeyBinding('x','Examine mode.',self.ExamineCallback)
        #self.vmtkRenderer.RenderWindowInteractor.AddObserver("KeyPressEvent", self.KeyPressed)

        self.SurfaceMapper = vtk.vtkPolyDataMapper()
        self.SurfaceMapper.SetInputData(self.Surface)
        self.SurfaceMapper.SetScalarVisibility(self.DisplayArray)
        surfaceActor = vtk.vtkActor()
        surfaceActor.SetMapper(self.SurfaceMapper)
        surfaceActor.GetProperty().SetOpacity(self.Opacity)
        self.vmtkRenderer.Renderer.AddActor(surfaceActor)

        self.ScalarBarActor = vtk.vtkScalarBarActor()
        self.ScalarBarActor.SetLookupTable(self.SurfaceMapper.GetLookupTable())
        self.ScalarBarActor.GetLabelTextProperty().ItalicOff()
        self.ScalarBarActor.GetLabelTextProperty().BoldOff()
        self.ScalarBarActor.GetLabelTextProperty().ShadowOff()
        self.ScalarBarActor.SetLabelFormat('%.2f')
        self.ScalarBarActor.SetTitle('distances')
        self.ScalarBarActor.VisibilityOff()
        self.vmtkRenderer.Renderer.AddActor(self.ScalarBarActor)

        self.SphereWidget = vtk.vtkSphereWidget()
        self.SphereWidget.SetInteractor(self.vmtkRenderer.RenderWindowInteractor)
        self.SphereWidget.AddObserver("InteractionEvent", self.SphereCallback)

        self.ExamineText = vtk.vtkTextActor()
        self.ExamineText.SetInput("Examine Mode")
        self.ExamineText.GetPositionCoordinate().SetCoordinateSystemToNormalizedViewport()
        self.ExamineText.SetPosition(0.05,0.95)
        self.ExamineText.VisibilityOff()
        self.vmtkRenderer.Renderer.AddActor2D(self.ExamineText)

        self.InputInfo('Please position the mouse and press space to add spheres, \'u\' to undo\n')

        any = 0
        while any == 0:
            self.InitializeSpheres()
            self.vmtkRenderer.Render()
            any = (self.Spheres.GetNumberOfPoints()>1)
            self.InputInfo('Please position the mouse and press space to add spheres, \'u\' to undo\nInsert at least 2 spheres.')

        self.Surface = self.ComputeArray()

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkcenterlineattributes as centerlineattributes


def test_abscissas(aorta_centerline, compare_centerlines):
    name = __name__ + '_test_abscissas.vtp'
    attributes = centerlineattributes.vmtkCenterlineAttributes()
    attributes.Centerlines = aorta_centerline
    attributes.Execute()

    assert compare_centerlines(attributes.Centerlines, name, method='addpointarray', arrayname='Abscissas') == True


def test_paralleltransportnormals(aorta_centerline, compare_centerlines):
    name = __name__ + '_tets_paralleltransportnormals.vtp'
    attributes = centerlineattributes.vmtkCenterlineAttributes()
    attributes.Centerlines = aorta_centerline
    attributes.Execute()

    assert compare_centerlines(attributes.Centerlines, name, method='addpointarray', arrayname='ParallelTransportNormals') == True



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkcenterlinemerge.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:49:59 $
## Version:   $Revision: 1.4 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes
from vmtk import vtkvmtk


class vmtkCenterlineMerge(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Centerlines = None

        self.RadiusArrayName = 'MaximumInscribedSphereRadius'
        self.GroupIdsArrayName = 'GroupIds'
        self.CenterlineIdsArrayName = 'CenterlineIds'
        self.TractIdsArrayName = 'TractIds'
        self.BlankingArrayName = 'Blanking'

        self.Length = 0.0
        self.MergeBlanked = 1

        self.SetScriptName('vmtkcenterlinemerge')
        self.SetScriptDoc('merge centerline tracts belonging to the same groups')
        self.SetInputMembers([
            ['Centerlines','i','vtkPolyData',1,'','the input centerlines','vmtksurfacereader'],
            ['RadiusArrayName','radiusarray','str',1,'','name of the array where centerline radius is stored'],
            ['GroupIdsArrayName','groupidsarray','str',1,'','name of the array where centerline group ids are stored'],
            ['CenterlineIdsArrayName','centerlineidsarray','str',1,'','name of the array where centerline ids are stored'],
            ['TractIdsArrayName','tractidsarray','str',1,'','name of the array where centerline tract ids are stored'],
            ['BlankingArrayName','blankingarray','str',1,'','name of the array where centerline blanking information about branches is stored'],
            ['Length','length','float',1,'(0.0,)','length of the resampling interval'],
            ['MergeBlanked','mergeblanked','bool',1,'','toggle generation of segments for blanked groups']
            ])
        self.SetOutputMembers([
            ['Centerlines','o','vtkPolyData',1,'','the output centerlines','vmtksurfacewriter']
            ])

    def Execute(self):

        if self.Centerlines == None:
            self.PrintError('Error: No input centerlines.')

        mergeCenterlines = vtkvmtk.vtkvmtkMergeCenterlines()
        mergeCenterlines.SetInputData(self.Centerlines)
        mergeCenterlines.SetRadiusArrayName(self.RadiusArrayName)
        mergeCenterlines.SetGroupIdsArrayName(self.GroupIdsArrayName)
        mergeCenterlines.SetCenterlineIdsArrayName(self.CenterlineIdsArrayName)
        mergeCenterlines.SetTractIdsArrayName(self.TractIdsArrayName)
        mergeCenterlines.SetBlankingArrayName(self.BlankingArrayName)
        mergeCenterlines.SetResamplingStepLength(self.Length)
        mergeCenterlines.SetMergeBlanked(self.MergeBlanked)
        mergeCenterlines.Update()

        self.Centerlines = mergeCenterlines.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkimageshiftscale.py,v $
## Language:  Python
## Date:      $Date: 2009/01/09 13:56:00 $
## Version:   $Revision: 1.0 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this class was contributed by
##       Hugo Gratama van Andel
##       Academic Medical Centre - University of Amsterdam
##       Dept. Biomedical Engineering  & Physics

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes


class vmtkImageShiftScale(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Image = None

        self.OutputType = 'unchanged'
        self.Shift = 0.0
        self.Scale = 1.0

        self.MapRanges = 0
        self.InputRange = [0.0,0.0]
        self.OutputRange = [0.0,0.0]

        self.ClampOverflowOn = 1

        self.SetScriptName('vmtkimageshiftscale')
        self.SetScriptDoc('shift and scale the intensity of an image and cast it to a specified type')
        self.SetInputMembers([
            ['Image','i','vtkImageData',1,'','the input image','vmtkimagereader'],
            ['OutputType','type','str',1,'["unchanged","float","double","uchar","char","ushort","short","long","ulong","int","uint"]','the output image type - use "unchanged", the default, to keep same type as input'],
            ['ClampOverflowOn','clamp','bool',1,'','Whith ClampOverflow On, the data is thresholded so that the output value does not exceed the max or min of the data type'],
            ['Shift','shift','float',1,'','the shift value'],
            ['Scale','scale','float',1,'','the scale value'],
            ['MapRanges','mapranges','bool',1,'','toggle mapping of input range to output range instead of simple shift scale'],
            ['InputRange','inputrange','float',2,'','the input range that will be mapped in the output range - leave default or set to 0.0 0.0 for using input image scalar range as input range'],
            ['OutputRange','outputrange','float',2,'','the output range into which the input range will be mapped']
            ])
        self.SetOutputMembers([
            ['Image','o','vtkImageData',1,'','the output image','vmtkimagewriter']
            ])

    def Execute(self):

        if self.Image == None:
            self.PrintError('Error: No input image.')

        if self.MapRanges:
            if self.InputRange == [0.0,0.0]:
                self.InputRange = self.Image.GetScalarRange()
            if self.InputRange[1] == self.InputRange[0]:
                self.PrintError('Error: Input range has zero width. Cannot map values')
            self.Shift = -self.InputRange[0]
            self.Scale = (self.OutputRange[1] - self.OutputRange[0]) / (self.InputRange[1] - self.InputRange[0])

        shiftScale = vtk.vtkImageShiftScale()
        shiftScale.SetInputData(self.Image)
        shiftScale.SetShift(self.Shift)
        shiftScale.SetScale(self.Scale)

        if self.OutputType == 'double':
            shiftScale.SetOutputScalarTypeToDouble()
        elif self.OutputType == 'uchar':
            shiftScale.SetOutputScalarTypeToUnsignedChar()
        elif self.OutputType == 'char':
            shiftScale.SetOutputScalarTypeToChar()
        elif self.OutputType == 'ushort':
            shiftScale.SetOutputScalarTypeToUnsignedShort()
        elif self.OutputType == 'short':
            shiftScale.SetOutputScalarTypeToShort()
        elif self.OutputType == 'ulong':
            shiftScale.SetOutputScalarTypeToUnsignedLong()
        elif self.OutputType == 'long':
            shiftScale.SetOutputScalarTypeToLong()
        elif self.OutputType == 'uint':
            shiftScale.SetOutputScalarTypeToUnsignedInt()
        elif self.OutputType == 'int':
            shiftScale.SetOutputScalarTypeToInt()
        elif self.OutputType == 'float':
            shiftScale.SetOutputScalarTypeToFloat()

        if self.OutputType != 'unchanged':
            if self.ClampOverflowOn:
                shiftScale.ClampOverflowOn()

        shiftScale.ClampOverflowOff()
        shiftScale.Update()

        self.Image = shiftScale.GetOutput()

        if self.MapRanges and self.OutputRange[0] != 0.0:
            shiftScale2 = vtk.vtkImageShiftScale()
            shiftScale2.SetInputData(self.Image)
            shiftScale2.SetShift(self.OutputRange[0])
            shiftScale2.SetScale(1.0)
            shiftScale2.Update()
            self.Image = shiftScale2.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      February 12, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkcenterlinesmoothing as centerlinesmoothing
from vtk.numpy_interface import dataset_adapter as dsa
import numpy as np


def test_default_params(aorta_centerline, compare_centerlines):
    name = __name__ + '_test_default_params.vtp'
    smoother = centerlinesmoothing.vmtkCenterlineSmoothing()
    smoother.Centerlines = aorta_centerline
    smoother.Execute()

    assert compare_centerlines(smoother.Centerlines, name) == True


@pytest.mark.parametrize("iterations,factor,paramid",[
    (100, 0.2, '0'),
    (200, 0.1, '1'),
    (200, 0.2, '2'),
    (50, 0.5, '3')
])
def test_varied_parameters(aorta_centerline, compare_centerlines,
                        iterations, factor, paramid):
    name = __name__ + '_test_varied_parameters_' + paramid + '.vtp'
    smoother = centerlinesmoothing.vmtkCenterlineSmoothing()
    smoother.Centerlines = aorta_centerline
    smoother.NumberOfSmoothingIterations = iterations
    smoother.SmoothingFactor = factor
    smoother.Execute()

    assert compare_centerlines(smoother.Centerlines, name) == True



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtklevelsetsegmentation.py,v $
## Language:  Python
## Date:      $Date: 2006/05/31 10:51:21 $
## Version:   $Revision: 1.19 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import sys
import math
import string
import vtk

from vmtk import vtkvmtk
from vmtk import pypes


class vmtkLevelSetSegmentation(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Image = None

        self.vmtkRenderer = None
        self.OwnRenderer = 0

        self.InitialLevelSets = None
        self.InitializationImage = None
        self.FeatureImage = None
        self.LevelSetsInput = None
        self.LevelSetsOutput = None
        self.LevelSets = None

        self.UpperThreshold = 0.0
        self.LowerThreshold = 0.0

        self.NumberOfIterations = 0
        self.PropagationScaling = 0.0
        self.CurvatureScaling = 0.0
        self.AdvectionScaling = 1.0

        self.IsoSurfaceValue = 0.0
        self.MaximumRMSError = 1E-20
        self.DerivativeSigma = 0.0
        self.FeatureDerivativeSigma = 0.0

        self.NegateForInitialization = 0

        self.SigmoidRemapping = 0

        self.LevelSetsType = 'geodesic'
        self.FeatureImageType = 'gradient'

        self.UpwindFactor = 1.0

        self.FWHMRadius = [1.0, 1.0, 1.0]
        self.FWHMBackgroundValue = 0.0

        self.EdgeWeight = 0.0
        self.SmoothingIterations = 5
        self.SmoothingTimeStep = 0.1
        self.SmoothingConductance = 0.8

        self.SetScriptName('vmtklevelsetsegmentation')
        self.SetScriptDoc('interactivly initialize an initial level set and evolve it to image gradients')
        self.SetInputMembers([
            ['Image','i','vtkImageData',1,'','','vmtkimagereader'],
            ['FeatureImage','featureimage','vtkImageData',1,'','','vmtkimagereader'],
            ['InitializationImage','initializationimage','vtkImageData',1,'','','vmtkimagereader'],
            ['InitialLevelSets','initiallevelsets','vtkImageData',1,'','','vmtkimagereader'],
            ['LevelSets','levelsets','vtkImageData',1,'','','vmtkimagereader'],
            ['LevelSetsType','levelsetstype','str',1,'["geodesic","curves","threshold","laplacian"]'],
            ['FeatureImageType','featureimagetype','str',1,'["vtkgradient","gradient","upwind","fwhm"]'],
            ['NegateForInitialization','negate','bool',1],
            ['SigmoidRemapping','sigmoid','bool',1],
            ['IsoSurfaceValue','isosurfacevalue','float',1],
            ['DerivativeSigma','derivativesigma','float',1,'(0.0,)'],
            ['FeatureDerivativeSigma','featurederivativesigma','float',1,'(0.0,)'],
            ['UpwindFactor','upwindfactor','float',1,'(0.0,1.0)'],
            ['FWHMRadius','fwhmradius','float',3,'(0.0,)'],
            ['FWHMBackgroundValue','fwhmbackgroundvalue','float',1],
            ['NumberOfIterations','iterations','int',1,'(0,)'],
            ['PropagationScaling','propagation','float',1,'(0.0,)'],
            ['CurvatureScaling','curvature','float',1,'(0.0,)'],
            ['AdvectionScaling','advection','float',1,'(0.0,)'],
            ['EdgeWeight','edgeweight','float',1,'(0.0,)'],
            ['SmoothingIterations','smoothingiterations','int',1,'(0,)'],
            ['SmoothingTimeStep','smoothingtimestep','float',1,'(0,)'],
            ['SmoothingConductance','smoothingconductance','float',1,'(0,)'],
            ['vmtkRenderer','renderer','vmtkRenderer',1]
            ])
        self.SetOutputMembers([
            ['LevelSets','o','vtkImageData',1,'','','vmtkimagewriter'],
            ['FeatureImage','ofeatureimage','vtkImageData',1,'','','vmtkimagewriter']
            ])

    def ThresholdValidator(self,text):
        if text == 'n':
            return 1
        if text == 'i':
            self.vmtkRenderer.Render()
            return 0
        try:
            float(text)
        except ValueError:
            return 0
        return 1

    def ThresholdInput(self,queryString):

        thresholdString = self.InputText(queryString,self.ThresholdValidator)

        threshold = None
        if thresholdString != 'n':
            threshold = float(thresholdString)

        return threshold

    def PrintProgress(self,obj,event):
        self.OutputProgress(obj.GetProgress(),10)

    def LevelSetEvolution(self):

        if self.LevelSetsType == "geodesic":
            levelSets = vtkvmtk.vtkvmtkGeodesicActiveContourLevelSetImageFilter()
            levelSets.SetFeatureImage(self.FeatureImage)
            levelSets.SetDerivativeSigma(self.FeatureDerivativeSigma)
            levelSets.SetAutoGenerateSpeedAdvection(1)
            levelSets.SetPropagationScaling(self.PropagationScaling)
            levelSets.SetCurvatureScaling(self.CurvatureScaling)
            levelSets.SetAdvectionScaling(self.AdvectionScaling)

        elif self.LevelSetsType == "curves":
            levelSets = vtkvmtk.vtkvmtkCurvesLevelSetImageFilter()
            levelSets.SetFeatureImage(self.FeatureImage)
            levelSets.SetDerivativeSigma(self.FeatureDerivativeSigma)
            levelSets.SetAutoGenerateSpeedAdvection(1)
            levelSets.SetPropagationScaling(self.PropagationScaling)
            levelSets.SetCurvatureScaling(self.CurvatureScaling)
            levelSets.SetAdvectionScaling(self.AdvectionScaling)

        elif self.LevelSetsType == "threshold":
            levelSets = vtkvmtk.vtkvmtkThresholdSegmentationLevelSetImageFilter()
            levelSets.SetFeatureImage(self.Image)
            queryString = "Please input lower threshold (\'n\' for none): "
            self.LowerThreshold = self.ThresholdInput(queryString)
            queryString = "Please input upper threshold (\'n\' for none): "
            self.UpperThreshold = self.ThresholdInput(queryString)
            scalarRange = self.Image.GetScalarRange()
            if self.LowerThreshold != None:
                levelSets.SetLowerThreshold(self.LowerThreshold)
            else:
                levelSets.SetLowerThreshold(scalarRange[0]-1.0)
            if self.UpperThreshold != None:
                levelSets.SetUpperThreshold(self.UpperThreshold)
            else:
                levelSets.SetUpperThreshold(scalarRange[1]+1.0)
            levelSets.SetEdgeWeight(self.EdgeWeight)
            levelSets.SetSmoothingIterations(self.SmoothingIterations)
            levelSets.SetSmoothingTimeStep(self.SmoothingTimeStep)
            levelSets.SetSmoothingConductance(self.SmoothingConductance)
            levelSets.SetPropagationScaling(self.PropagationScaling)
            levelSets.SetCurvatureScaling(self.CurvatureScaling)

        elif self.LevelSetsType == "laplacian":
            levelSets = vtkvmtk.vtkvmtkLaplacianSegmentationLevelSetImageFilter()
            levelSets.SetFeatureImage(self.Image)
            levelSets.SetPropagationScaling(-self.PropagationScaling)
            levelSets.SetCurvatureScaling(self.CurvatureScaling)

        else:
            self.PrintError('Unsupported LevelSetsType')

        levelSets.SetInputData(self.LevelSetsInput)
        levelSets.SetNumberOfIterations(self.NumberOfIterations)
        levelSets.SetIsoSurfaceValue(self.IsoSurfaceValue)
        levelSets.SetMaximumRMSError(self.MaximumRMSError)
        levelSets.SetInterpolateSurfaceLocation(1)
        levelSets.SetUseImageSpacing(1)
        levelSets.AddObserver("ProgressEvent", self.PrintProgress)
        levelSets.Update()

        self.EndProgress()

        self.LevelSetsOutput = vtk.vtkImageData()
        self.LevelSetsOutput.DeepCopy(levelSets.GetOutput())

    def MergeLevelSet(self):

        if self.LevelSets == None:
            self.LevelSets = self.LevelSetsOutput
        else:
            minFilter = vtk.vtkImageMathematics()
            minFilter.SetOperationToMin()
            minFilter.SetInput1Data(self.LevelSets)
            minFilter.SetInput2Data(self.LevelSetsOutput)
            minFilter.Update()
            self.LevelSets = minFilter.GetOutput()

    def DisplayLevelSetSurface(self,levelSets,value=0.0):

        marchingCubes = vtk.vtkMarchingCubes()
        marchingCubes.SetInputData(levelSets)
        marchingCubes.SetValue(0,value)
        marchingCubes.Update()

        self.OutputText('Displaying.\n')

        self.SurfaceViewer.Surface = marchingCubes.GetOutput()
        self.SurfaceViewer.Display = 0
        self.SurfaceViewer.Opacity = 0.5
        self.SurfaceViewer.BuildView()

    def YesNoValidator(self,text):
        if text in ['n','y']:
            return 1
        return 0

    def EvolutionParametersValidator(self,text):
        if not text:
            return 1
        if text in ['q','e']:
            return 1
        splitText = text.strip().split(' ')
        if len(splitText) not in [1,4]:
            return 0
        try:
            int(splitText[0])
            if len(splitText) == 4:
                float(splitText[1])
                float(splitText[2])
                float(splitText[3])
        except ValueError:
            return 0
        return 1

    def Execute(self):
        from vmtk import vmtkscripts
        if self.Image == None:
            self.PrintError('Error: no Image.')

        if self.Image.GetScalarTypeAsString() != 'float':
            cast = vtk.vtkImageCast()
            cast.SetInputData(self.Image)
            cast.SetOutputScalarTypeToFloat()
            cast.Update()
            self.Image = cast.GetOutput()

        if not self.InitializationImage:
            self.InitializationImage = self.Image

        if not self.FeatureImage:
            if self.LevelSetsType in ["geodesic", "curves"]:
                imageFeatures = vmtkscripts.vmtkImageFeatures()
                imageFeatures.Image = self.Image
                imageFeatures.FeatureImageType = self.FeatureImageType
                imageFeatures.SigmoidRemapping = self.SigmoidRemapping
                imageFeatures.DerivativeSigma = self.FeatureDerivativeSigma
                imageFeatures.UpwindFactor = self.UpwindFactor
                imageFeatures.FWHMRadius = self.FWHMRadius
                imageFeatures.FWHMBackgroundValue = self.FWHMBackgroundValue
                imageFeatures.Execute()
                self.FeatureImage = imageFeatures.FeatureImage
            elif self.LevelSetsType in ["threshold", "laplacian"]:
                self.FeatureImage = self.Image
            else:
                self.PrintError('Unsupported LevelSetsType')

        if self.NumberOfIterations != 0:
            self.LevelSetsInput = self.InitialLevelSets
            self.LevelSetEvolution()
            self.MergeLevelSet()
            return

        if not self.vmtkRenderer:
            self.vmtkRenderer = vmtkscripts.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.vmtkRenderer.RegisterScript(self)

        self.ImageSeeder = vmtkscripts.vmtkImageSeeder()
        self.ImageSeeder.vmtkRenderer = self.vmtkRenderer
        #self.ImageSeeder.Image = self.Image
        self.ImageSeeder.Image = self.InitializationImage
        self.ImageSeeder.Display = 0
        self.ImageSeeder.Execute()
        ##self.ImageSeeder.Display = 1
        self.ImageSeeder.BuildView()

        self.SurfaceViewer = vmtkscripts.vmtkSurfaceViewer()
        self.SurfaceViewer.vmtkRenderer = self.vmtkRenderer

        if self.LevelSets != None:
            self.DisplayLevelSetSurface(self.LevelSets,0.0)

        self.vmtkImageInitialization = vmtkscripts.vmtkImageInitialization()
        #self.vmtkImageInitialization.Image = self.Image
        self.vmtkImageInitialization.Image = self.InitializationImage
        self.vmtkImageInitialization.vmtkRenderer = self.vmtkRenderer
        self.vmtkImageInitialization.ImageSeeder = self.ImageSeeder
        self.vmtkImageInitialization.SurfaceViewer = self.SurfaceViewer
        self.vmtkImageInitialization.NegateImage = self.NegateForInitialization
        self.vmtkImageInitialization.OwnRenderer = 0

        endSegmentation = 0
        while (endSegmentation == 0):

            if self.InitialLevelSets == None:
                self.vmtkImageInitialization.Execute()
                self.LevelSetsInput = self.vmtkImageInitialization.InitialLevelSets
#                self.IsoSurfaceValue = self.vmtkImageInitialization.IsoSurfaceValue
                self.vmtkImageInitialization.InitialLevelSets = None
#                self.vmtkImageInitialization.IsosurfaceValue = 0.0
                self.IsoSurfaceValue = 0.0
            else:
                self.LevelSetsInput = self.InitialLevelSets
                self.InitialLevelSets = None
                self.DisplayLevelSetSurface(self.LevelSetsInput,self.IsoSurfaceValue)

            endEvolution = False
            while not endEvolution:

                queryString = 'Please input parameters (type return to accept current values, \'e\' to end, \'q\' to quit):\nNumberOfIterations('+str(self.NumberOfIterations)+') [PropagationScaling('+str(self.PropagationScaling)+') CurvatureScaling('+str(self.CurvatureScaling)+') AdvectionScaling('+str(self.AdvectionScaling)+')]: '
                inputString = self.InputText(queryString,self.EvolutionParametersValidator)

                if inputString == 'q':
                    return
                elif inputString == 'e':
                    endEvolution = True
                elif inputString != '':
                    splitInputString = inputString.strip().split(' ')
                    if len(splitInputString) == 1:
                        self.NumberOfIterations = int(splitInputString[0])
                    elif len(splitInputString) == 4:
                        self.NumberOfIterations = int(splitInputString[0])
                        self.PropagationScaling = float(splitInputString[1])
                        self.CurvatureScaling = float(splitInputString[2])
                        self.AdvectionScaling = float(splitInputString[3])
                    else:
                        self.PrintLog('Wrong number of parameters.')
                        continue

                if endEvolution:
                    break

                self.LevelSetEvolution()
                self.DisplayLevelSetSurface(self.LevelSetsOutput)

                queryString = 'Accept result? (y/n): '
                inputString = self.InputText(queryString,self.YesNoValidator)
                if inputString == 'y':
                    endEvolution = True
                elif inputString == 'n':
                    endEvolution = False

            queryString = 'Merge branch? (y/n): '
            inputString = self.InputText(queryString,self.YesNoValidator)
            if inputString == 'y':
                self.MergeLevelSet()
            elif inputString == 'n':
                pass

            if self.LevelSets != None:
                self.DisplayLevelSetSurface(self.LevelSets)

            queryString = 'Segment another branch? (y/n): '
            inputString = self.InputText(queryString,self.YesNoValidator)
            if inputString == 'y':
                endSegmentation = False
            elif inputString == 'n':
                endSegmentation = True
        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfacesmoothing.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:53:14 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes


class vmtkSurfaceSmoothing(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None

        self.NumberOfIterations = 1
        self.PassBand = 1.0
        self.RelaxationFactor = 0.01
        self.BoundarySmoothing = 1
        self.NormalizeCoordinates = 1

        self.Method = 'taubin'

        self.SetScriptName('vmtksurfacesmoothing')
        self.SetScriptDoc('smooth a surface using Taubin\'s algorithm')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['NumberOfIterations','iterations','int',1,'(0,)','number of iterations (e.g. 25)'],
            ['Method','method','str',1,'["taubin","laplace"]','smoothing method'],
            ['PassBand','passband','float',1,'','pass band (e.g. 0.1) - taubin only'],
            ['RelaxationFactor','relaxation','float',1,'(0.0,)','relaxation factor (e.g. 0.01) - laplace only'],
            ['BoundarySmoothing','boundarysmoothing','bool',1,'','toggle allow change of position of boundary points'],
            ['NormalizeCoordinates','normalize','bool',1,'','toggle normalization of coordinates prior to filtering to minimize spurious translation effects - taubin only']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter']
            ])

    def Execute(self):

        from vmtk import vmtkscripts
        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        smoothingFilter = None

        if self.Method == 'taubin':
            smoothingFilter = vtk.vtkWindowedSincPolyDataFilter()
            smoothingFilter.SetInputData(self.Surface)
            smoothingFilter.SetNumberOfIterations(self.NumberOfIterations)
            smoothingFilter.SetPassBand(self.PassBand)
            smoothingFilter.SetBoundarySmoothing(self.BoundarySmoothing)
            smoothingFilter.SetNormalizeCoordinates(self.NormalizeCoordinates)
            smoothingFilter.Update()
        elif self.Method == 'laplace':
            smoothingFilter = vtk.vtkSmoothPolyDataFilter()
            smoothingFilter.SetInputData(self.Surface)
            smoothingFilter.SetNumberOfIterations(self.NumberOfIterations)
            smoothingFilter.SetRelaxationFactor(self.RelaxationFactor)
            smoothingFilter.Update()
        else:
            self.PrintError('Error: smoothing method not supported.')

        self.Surface = smoothingFilter.GetOutput()

        normals = vmtkscripts.vmtkSurfaceNormals()
        normals.Surface = self.Surface
        normals.Execute()

        self.Surface = normals.Surface


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import unittest
import os


@pytest.fixture(scope='function')
def vmtk_vtkvmtk_attributes():
    attributes =  [
        'vtk',
        'vtkvmtkActiveTubeFilter',
        'vtkvmtkAnisotropicDiffusionImageFilter',
        'vtkvmtkAnnularCapPolyData',
        'vtkvmtkAppendFilter',
        'vtkvmtkBoundaryConditions',
        'vtkvmtkBoundaryLayerGenerator',
        # 'vtkvmtkBoundaryLayerGenerator2',
        'vtkvmtkBoundaryReferenceSystems',
        'vtkvmtkBoundedReciprocalImageFilter',
        'vtkvmtkCapPolyData',
        'vtkvmtkCardinalSpline',
        # 'vtkvmtkCellDimensionFilter',
        'vtkvmtkCenterlineAttributesFilter',
        'vtkvmtkCenterlineBifurcationReferenceSystems',
        'vtkvmtkCenterlineBifurcationVectors',
        'vtkvmtkCenterlineBranchExtractor',
        'vtkvmtkCenterlineBranchGeometry',
        'vtkvmtkCenterlineEndpointExtractor',
        'vtkvmtkCenterlineGeometry',
        # 'vtkvmtkCenterlineInterpolateArray',
        'vtkvmtkCenterlineReferenceSystemAttributesOffset',
        'vtkvmtkCenterlineSmoothing',
        'vtkvmtkCenterlineSphereDistance',
        'vtkvmtkCenterlineSplitExtractor',
        'vtkvmtkCenterlineSplittingAndGroupingFilter',
        'vtkvmtkCenterlineUtilities',
        'vtkvmtkCollidingFrontsImageFilter',
        # 'vtkvmtkConcaveAnnularCapPolyData',
        'vtkvmtkCurvedMPRImageFilter',
        'vtkvmtkCurvesLevelSetImageFilter',
        'vtkvmtkDanielssonDistanceMapImageFilter',
        'vtkvmtkDataSetItem',
        'vtkvmtkDataSetItems',
        'vtkvmtkDirichletBoundaryConditions',
        'vtkvmtkDolfinWriter',
        # 'vtkvmtkDolfinWriter2',
        'vtkvmtkDoubleVector',
        'vtkvmtkEllipticProblem',
        'vtkvmtkEmptyNeighborhood',
        'vtkvmtkEmptyStencil',
        'vtkvmtkFDNEUTReader',
        'vtkvmtkFDNEUTWriter',
        'vtkvmtkFEAssembler',
        'vtkvmtkFEShapeFunctions',
        'vtkvmtkFWHMFeatureImageFilter',
        'vtkvmtkFastMarchingDirectionalFreezeImageFilter',
        'vtkvmtkFastMarchingUpwindGradientImageFilter',
        'vtkvmtkFluentWriter',
        'vtkvmtkGaussQuadrature',
        'vtkvmtkGeodesicActiveContourLevelSet2DImageFilter',
        'vtkvmtkGeodesicActiveContourLevelSetImageFilter',
        'vtkvmtkGradientMagnitudeImageFilter',
        'vtkvmtkGradientMagnitudeRecursiveGaussian2DImageFilter',
        'vtkvmtkGradientMagnitudeRecursiveGaussianImageFilter',
        'vtkvmtkGrayscaleMorphologyImageFilter',
        'vtkvmtkITKArchetypeImageSeriesReader',
        'vtkvmtkITKArchetypeImageSeriesScalarReader',
        'vtkvmtkITKImageWriter',
        'vtkvmtkITKVersion',
        'vtkvmtkImageBoxPainter',
        'vtkvmtkImagePlaneWidget',
        'vtkvmtkInteractorStyleTrackballCamera',
        'vtkvmtkInternalTetrahedraExtractor',
        'vtkvmtkItem',
        'vtkvmtkItems',
        'vtkvmtkIterativeClosestPointTransform',
        'vtkvmtkLaplacianSegmentationLevelSetImageFilter',
        'vtkvmtkLevelSetSigmoidFilter',
        'vtkvmtkLinearSystem',
        'vtkvmtkLinearSystemSolver',
        'vtkvmtkLinearToQuadraticMeshFilter',
        'vtkvmtkLinearToQuadraticSurfaceMeshFilter',
        'vtkvmtkLinearizeMeshFilter',
        'vtkvmtkMath',
        'vtkvmtkMedialCurveFilter',
        'vtkvmtkMergeCenterlines',
        'vtkvmtkMeshLambda2',
        'vtkvmtkMeshProjection',
        'vtkvmtkMeshVelocityStatistics',
        'vtkvmtkMeshVorticity',
        'vtkvmtkMeshWallShearRate',
        'vtkvmtkMinHeap',
        'vtkvmtkNeighborhood',
        'vtkvmtkNeighborhoods',
        'vtkvmtkNonManifoldFastMarching',
        'vtkvmtkNonManifoldSteepestDescent',
        'vtkvmtkNormalizeImageFilter',
        'vtkvmtkObjectnessMeasureImageFilter',
        'vtkvmtkOpenNLLinearSystemSolver',
        'vtkvmtkOtsuMultipleThresholdsImageFilter',
        'vtkvmtkPNGWriter',
        'vtkvmtkPolyBall',
        'vtkvmtkPolyBallLine',
        # 'vtkvmtkPolyBallLine2',
        'vtkvmtkPolyBallModeller',
        'vtkvmtkPolyDataAreaWeightedUmbrellaStencil',
        'vtkvmtkPolyDataBifurcationProfiles',
        'vtkvmtkPolyDataBifurcationSections',
        'vtkvmtkPolyDataBoundaryExtractor',
        'vtkvmtkPolyDataBranchSections',
        'vtkvmtkPolyDataBranchUtilities',
        'vtkvmtkPolyDataCenterlineAbscissaMetricFilter',
        'vtkvmtkPolyDataCenterlineAngularMetricFilter',
        'vtkvmtkPolyDataCenterlineGroupsClipper',
        'vtkvmtkPolyDataCenterlineMetricFilter',
        'vtkvmtkPolyDataCenterlineProjection',
        'vtkvmtkPolyDataCenterlineSections',
        'vtkvmtkPolyDataCenterlines',
        'vtkvmtkPolyDataClampedSmoothingFilter',
        'vtkvmtkPolyDataCylinderHarmonicMappingFilter',
        # 'vtkvmtkPolyDataDijkstraDistanceToPoints',
        'vtkvmtkPolyDataDiscreteElasticaFilter',
        'vtkvmtkPolyDataDistanceToCenterlines',
        # 'vtkvmtkPolyDataDistanceToSpheres',
        'vtkvmtkPolyDataFEGradientAssembler',
        'vtkvmtkPolyDataFELaplaceAssembler',
        'vtkvmtkPolyDataFELaplaceBeltramiStencil',
        'vtkvmtkPolyDataFVFELaplaceBeltramiStencil',
        'vtkvmtkPolyDataFlowExtensionsFilter',
        # 'vtkvmtkPolyDataGeodesicRBFInterpolation',
        'vtkvmtkPolyDataGradientFilter',
        'vtkvmtkPolyDataGradientStencil',
        'vtkvmtkPolyDataHarmonicMappingFilter',
        'vtkvmtkPolyDataKiteRemovalFilter',
        'vtkvmtkPolyDataLaplaceBeltramiStencil',
        'vtkvmtkPolyDataLineEmbedder',
        'vtkvmtkPolyDataLocalGeometry',
        'vtkvmtkPolyDataManifoldExtendedNeighborhood',
        'vtkvmtkPolyDataManifoldNeighborhood',
        'vtkvmtkPolyDataManifoldStencil',
        'vtkvmtkPolyDataMeanCurvature',
        'vtkvmtkPolyDataMultipleCylinderHarmonicMappingFilter',
        'vtkvmtkPolyDataNeighborhood',
        'vtkvmtkPolyDataNetworkExtraction',
        'vtkvmtkPolyDataNormalPlaneEstimator',
        'vtkvmtkPolyDataPatchingFilter',
        'vtkvmtkPolyDataPotentialFit',
        'vtkvmtkPolyDataReferenceSystemBoundaryMetricFilter',
        'vtkvmtkPolyDataRigidSurfaceModelling',
        # 'vtkvmtkPolyDataSampleFunction',
        'vtkvmtkPolyDataScissors',
        'vtkvmtkPolyDataSizingFunction',
        'vtkvmtkPolyDataStencilFlowFilter',
        'vtkvmtkPolyDataStretchMappingFilter',
        'vtkvmtkPolyDataSurfaceRemeshing',
        'vtkvmtkPolyDataToUnstructuredGridFilter',
        'vtkvmtkPolyDataUmbrellaStencil',
        'vtkvmtkRBFInterpolation',
        # 'vtkvmtkRBFInterpolation2',
        'vtkvmtkRecursiveGaussian2DImageFilter',
        'vtkvmtkRecursiveGaussianImageFilter',
        'vtkvmtkReferenceSystemUtilities',
        'vtkvmtkSatoVesselnessMeasureImageFilter',
        'vtkvmtkSigmoidImageFilter',
        'vtkvmtkSimpleCapPolyData',
        'vtkvmtkSimplifyVoronoiDiagram',
        'vtkvmtkSmoothCapPolyData',
        'vtkvmtkSparseMatrix',
        'vtkvmtkSparseMatrixRow',
        'vtkvmtkStaticTemporalInterpolatedVelocityField',
        'vtkvmtkStaticTemporalStreamTracer',
        'vtkvmtkSteepestDescentLineTracer',
        'vtkvmtkSteepestDescentShooter',
        'vtkvmtkStencil',
        'vtkvmtkStencils',
        'vtkvmtkStreamlineClusteringFilter',
        'vtkvmtkStreamlineOsculatingCentersFilter',
        'vtkvmtkStreamlineToParticlesFilter',
        'vtkvmtkSurfaceDistance',
        # 'vtkvmtkSurfaceProjectCellArray',
        'vtkvmtkSurfaceProjection',
        'vtkvmtkTetGenReader',
        'vtkvmtkTetGenWrapper',
        'vtkvmtkTetGenWriter',
        'vtkvmtkThresholdSegmentationLevelSetImageFilter',
        'vtkvmtkTopologicalSeamFilter',
        'vtkvmtkUnstructuredGridCenterlineGroupsClipper',
        'vtkvmtkUnstructuredGridCenterlineSections',
        'vtkvmtkUnstructuredGridFEGradientAssembler',
        'vtkvmtkUnstructuredGridFELaplaceAssembler',
        'vtkvmtkUnstructuredGridFEVorticityAssembler',
        'vtkvmtkUnstructuredGridGradientFilter',
        'vtkvmtkUnstructuredGridHarmonicMappingFilter',
        'vtkvmtkUnstructuredGridNeighborhood',
        'vtkvmtkUnstructuredGridTetraFilter',
        'vtkvmtkUnstructuredGridVorticityFilter',
        'vtkvmtkUpwindGradientMagnitudeImageFilter',
        'vtkvmtkVesselEnhancingDiffusionImageFilter',
        'vtkvmtkVesselnessMeasureImageFilter',
        'vtkvmtkVoronoiDiagram3D',
        'vtkvmtkXdaReader',
        'vtkvmtkXdaWriter'
        ]
    return attributes


def test_import_vtkvmtk(vmtk_vtkvmtk_attributes):
    from vmtk import vtkvmtk
    print(vmtk_vtkvmtk_attributes)
    for name in vmtk_vtkvmtk_attributes:
        assert hasattr(vtkvmtk, name)
        # assert importlib.__import__(name)


def test_itk_version():
    from vmtk.vtkvmtk import vtkvmtkITKVersion
    print(vtkvmtkITKVersion.GetITKVersion())
    print(vtkvmtkITKVersion.GetITKMajorVersion())
    print(vtkvmtkITKVersion.GetITKMinorVersion())
    print(vtkvmtkITKVersion.GetITKBuildVersion())
    print(vtkvmtkITKVersion.GetITKSourceVersion())



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkmeshtetrahedralize.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:53:14 $
## Version:   $Revision: 1.1 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes

from vmtk import vtkvmtk


class vmtkMeshTetrahedralize(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Mesh = None
        self.TetrahedraOnly = 0

        self.SetScriptName('vmtkmeshtetrahedralize')
        self.SetScriptDoc('convert the elements of a mesh to linear')
        self.SetInputMembers([
            ['Mesh','i','vtkUnstructuredGrid',1,'','the input mesh','vmtkmeshreader'],
            ['TetrahedraOnly','tetonly','bool',1,'','toggle suppression of 1D and 2D cells']
            ])
        self.SetOutputMembers([
            ['Mesh','o','vtkUnstructuredGrid',1,'','the output mesh','vmtkmeshwriter']])

    def Execute(self):

        if self.Mesh == None:
            self.PrintError('Error: No input mesh.')

        tetrahedralizeFilter = vtkvmtk.vtkvmtkUnstructuredGridTetraFilter()
        tetrahedralizeFilter.SetInputData(self.Mesh)
        tetrahedralizeFilter.SetTetrahedraOnly(self.TetrahedraOnly)
        tetrahedralizeFilter.Update()

        self.Mesh = tetrahedralizeFilter.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfmesh.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:49:59 $
## Version:   $Revision: 1.6 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vmtk import vtkvmtk
import sys

from vmtk import pypes


class vmtkSurfMesh(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.NodeSpacing = 1.0

        self.SetScriptName('vmtksurfmesh')
        self.SetScriptDoc('wrapper around surfmesh surface mesh generator by Gordan Stuhne')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['NodeSpacing','nodespacing','float',1,'(0.0,)','desired node spacing']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter']
            ])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No Surface.')

        surfmesh = vtkvmtk.vtkvmtkSurfMeshWrapper()
        surfmesh.SetInputData(self.Surface)
        surfmesh.SetNodeSpacing(self.NodeSpacing)
        surfmesh.Update()

        self.Surface = surfmesh.GetOutput()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkmeshmergetimesteps.py,v $
## Language:  Python
## Date:      $Date: 2013/07/15 12:59:27 $
## Version:   $Revision: 1.6 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this class was contributed by
##       Simone Manini
##       Orobix Srl

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys
import os

from vmtk import vtkvmtk
from vmtk import vmtkmeshreader
from vmtk import vmtkmeshvectorfromcomponents
from vmtk import pypes


class vmtkMeshMergeTimesteps(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Mesh = None
        self.InputDirectoryName = None
        self.Pattern = None
        self.FirstTimeStep = None
        self.LastTimeStep = None
        self.IntervalTimeStep = 1
        self.VelocityVector = 0
        self.VelocityVectorArrayName = None
        self.Pressure = 0
        self.PressureArrayName = 'p'
        self.Wsr = 0
        self.VelocityComponentsArrayNames = 'u v w'
        self.WsrComponentsArrayNames = 'taux tauy tauz'
        self.WsrVector = 0
        self.SetScriptName('vmtkmeshmergetimesteps')
        self.SetScriptDoc('merge multiple mesh files with different timesteps into one')

        self.SetInputMembers([
            ['InputDirectoryName','directory','str',1,''],
            ['Pattern','pattern','str',1,''],
            ['FirstTimeStep','firststep','int',1,'(0,)'],
            ['LastTimeStep','laststep','int',1,'(0,)'],
            ['IntervalTimeStep','intervalstep','int',1,'(0,)'],
            ['VelocityComponentsArrayNames','components','str',-1,'',''],
            ['VelocityVector','velocityvector','bool',1,'','velocity vector instead of components'],
            ['VelocityVectorArrayName','vector','str',1,'','velocity vector name'],
            ['Pressure','pressure','bool',1,'','pressure array'],
            ['PressureArrayName','pressurearrayname','str',1,'','name of the pressure array'],
            ['Wsr','wallshearrate','bool',1,'','wallshearrate array'],
            ['WsrComponentsArrayNames','components','str',-1,'',''],
            ['WsrVector','wsrvector','bool',1,'','wallshearrate vector instead of components'],
            ])
        self.SetOutputMembers([
            ['Mesh','o','vtkUnstructuredGrid',1,'','the output mesh','vmtkmeshwriter']
            ])

    def Execute(self):

        if (self.InputDirectoryName == None):
            self.PrintError('Error: no directory.')

        if (self.Pattern == None):
            self.PrintError('Error: no pattern.')

        if (self.FirstTimeStep == None):
            self.PrintError('Error: no first timestep.')

        if (self.LastTimeStep == None):
            self.PrintError('Error: no last timestep.')

        if (self.VelocityComponentsArrayNames == None):
            self.PrintError('Error: no VelocityComponentsArrayNames.')

        for root, dirs, files in os.walk(self.InputDirectoryName):
            if root == self.InputDirectoryName:
                fileList = [x for x in files if not (x.startswith('.'))]

        timeIndexList = list(range(self.FirstTimeStep,self.LastTimeStep+1,self.IntervalTimeStep))
        reader = vmtkmeshreader.vmtkMeshReader()
        #if self.VelocityVector or self.WsrVector:
        if self.WsrVector:
            vectorFromComponents = vmtkmeshvectorfromcomponents.vmtkMeshVectorFromComponents()

        u_name = self.VelocityComponentsArrayNames.split(' ')[0]
        v_name = self.VelocityComponentsArrayNames.split(' ')[1]
        w_name = self.VelocityComponentsArrayNames.split(' ')[2]

        if self.Wsr:
            taux_name = self.WsrComponentsArrayNames.split(' ')[0]
            tauy_name = self.WsrComponentsArrayNames.split(' ')[1]
            tauz_name = self.WsrComponentsArrayNames.split(' ')[2]

        field = vtk.vtkFieldData()
        field.AllocateArrays(1)
        timesteps = vtk.vtkIntArray()
        timesteps.SetNumberOfComponents(1)
        timesteps.SetName("timesteps")
        i = 0
        for step in timeIndexList:
            if (self.Pattern%step).replace(' ','0') in fileList:
                timesteps.InsertTuple1(i, step)
                i+=1

                fileName = (self.Pattern%step).replace(' ','0')
                timeIndex = step
                reader.InputFileName = os.path.abspath(os.path.join(self.InputDirectoryName,fileName))
                reader.Execute()
                mesh = reader.Mesh

                if step == self.FirstTimeStep:
                    mesh.CopyStructure(mesh)
                    self.Mesh = mesh

                if self.Pressure:
                    p = mesh.GetPointData().GetArray(self.PressureArrayName)
                    p.SetName(self.PressureArrayName+str(step))
                    self.Mesh.GetPointData().AddArray(p)

                if self.Wsr:
                    taux = mesh.GetPointData().GetArray(taux_name)
                    taux.SetName(taux_name+str(step))
                    self.Mesh.GetPointData().AddArray(taux)

                    tauy = mesh.GetPointData().GetArray(tauy_name)
                    tauy.SetName(tauy_name+str(step))
                    self.Mesh.GetPointData().AddArray(tauy)

                    tauz = mesh.GetPointData().GetArray(tauz_name)
                    tauz.SetName(tauz_name+str(step))
                    self.Mesh.GetPointData().AddArray(tauz)

                if self.VelocityVector:

                  j = 0

                  u_component = vtk.vtkDoubleArray()
                  u_component.SetNumberOfComponents(1)

                  v_component = vtk.vtkDoubleArray()
                  v_component.SetNumberOfComponents(1)

                  w_component = vtk.vtkDoubleArray()
                  w_component.SetNumberOfComponents(1)

                  while j < mesh.GetPointData().GetArray(self.VelocityVectorArrayName).GetNumberOfTuples():

                    u_val = mesh.GetPointData().GetArray(self.VelocityVectorArrayName).GetComponent(j,0)
                    v_val = mesh.GetPointData().GetArray(self.VelocityVectorArrayName).GetComponent(j,1)
                    w_val = mesh.GetPointData().GetArray(self.VelocityVectorArrayName).GetComponent(j,2)

                    u_component.InsertTuple1(j, u_val)
                    v_component.InsertTuple1(j, v_val)
                    w_component.InsertTuple1(j, w_val)

                    j+=1

                  u_component.SetName(u_name+"_"+str(step))
                  self.Mesh.GetPointData().AddArray(u_component)

                  v_component.SetName(v_name+"_"+str(step))
                  self.Mesh.GetPointData().AddArray(v_component)

                  w_component.SetName(w_name+"_"+str(step))
                  self.Mesh.GetPointData().AddArray(w_component)

                else:

                  u = mesh.GetPointData().GetArray(u_name)
                  u.SetName(u_name+"_"+str(step))
                  self.Mesh.GetPointData().AddArray(u)

                  v = mesh.GetPointData().GetArray(v_name)
                  v.SetName(v_name+"_"+str(step))
                  self.Mesh.GetPointData().AddArray(v)

                  w = mesh.GetPointData().GetArray(w_name)
                  w.SetName(w_name+"_"+str(step))
                  self.Mesh.GetPointData().AddArray(w)

                if self.WsrVector:
                    vectorFromComponents.Mesh = self.Mesh
                    vectorFromComponents.VectorArrayName = "Wsr_"+str(step)
                    vectorFromComponents.ComponentsArrayNames = [taux.GetName(),tauy.GetName(),tauz.GetName()]
                    vectorFromComponents.RemoveComponentArrays = True
                    vectorFromComponents.Execute()

        field.AddArray(timesteps)
        self.Mesh.SetFieldData(field)


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python
""" This module loads all the classes from the vtkVmtk library into its
namespace.  This is a required module."""

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import os
import vtk

from .vtkvmtkCommonPython import *
from .vtkvmtkComputationalGeometryPython import *
from .vtkvmtkDifferentialGeometryPython import *
from .vtkvmtkIOPython import *
from .vtkvmtkMiscPython import *
try:
  from .vtkvmtkRenderingPython import *
except:
  pass
from .vtkvmtkSegmentationPython import *
from .vtkvmtkITKPython import *
try:
  from .vtkvmtkContribPython import *
except:
  pass



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkmeshconnectivity.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:49:59 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes


class vmtkMeshConnectivity(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Mesh = None
        self.ReferenceMesh = None

        self.ClosestPoint = None

        self.Method = 'largest'

        self.SetScriptName('vmtkmeshconnectivity')
        self.SetScriptDoc('extract the largest connected region or the closest point-connected region from a mesh')
        self.SetInputMembers([
            ['Mesh','i','vtkUnstructuredGrid',1,'','the input mesh','vmtkmeshreader'],
            ['Method','method','str',1,'["largest","closest"]','connectivity method'],
            ['ClosestPoint','closestpoint','float',3,'','coordinates of the closest point'],
            ['ReferenceMesh','r','vtkUnstructuredGrid',1,'','the reference mesh, whose barycenter will be used as closest point for the connectivity filter','vmtkmeshreader']
            ])
        self.SetOutputMembers([
            ['Mesh','o','vtkUnstructuredGrid',1,'','the output mesh','vmtkmeshwriter']
            ])

    def Execute(self):

        if self.Mesh == None:
            self.PrintError('Error: No input mesh.')

        barycenter = [0.0,0.0,0.0]
        if self.Method == 'closest' and self.ClosestPoint == None:
            n = self.ReferenceMesh.GetNumberOfPoints()
            for i in range(n):
                point = self.ReferenceMesh.GetPoint(i)
                barycenter[0] += point[0]
                barycenter[1] += point[1]
                barycenter[2] += point[2]
            barycenter[0] /= n
            barycenter[1] /= n
            barycenter[2] /= n

        connectivityFilter = vtk.vtkConnectivityFilter()
        connectivityFilter.SetInputData(self.Mesh)
        connectivityFilter.ColorRegionsOff()
        if self.Method == 'largest':
            connectivityFilter.SetExtractionModeToLargestRegion()
        elif self.Method == 'closest':
            connectivityFilter.SetExtractionModeToClosestPointRegion()
            if self.ClosestPoint:
                connectivityFilter.SetClosestPoint(self.ClosestPoint)
            else:
                connectivityFilter.SetClosestPoint(barycenter)
        connectivityFilter.Update()

        self.Mesh = connectivityFilter.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfaceclipper.py,v $
## Language:  Python
## Date:      $Date: 2006/05/26 12:35:13 $
## Version:   $Revision: 1.9 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import vmtkrenderer
from vmtk import pypes


class vmtkSurfaceClipper(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.ClippedSurface = None
        self.CutLines = None
        self.vmtkRenderer = None
        self.OwnRenderer = 0

        self.WidgetType = 'box'

        self.Actor = None
        self.ClipWidget = None
        self.ClipFunction = None
        self.CleanOutput = 1
        self.Transform = None

        self.InsideOut = 0

        self.Interactive = 1
        self.ClipArrayName = None
        self.ClipValue = 0.0

        self.SetScriptName('vmtksurfaceclipper')
        self.SetScriptDoc('interactively clip a surface with a box')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['WidgetType','type','str',1,'["box","sphere"]','type of widget used for clipping'],
            ['Transform','transform','vtkTransform',1,'','the widget transform, useful in case of piping of multiple clipping scripts'],
            ['CleanOutput','cleanoutput','bool',1,'','toggle cleaning the unused points'],
            ['InsideOut','insideout','bool',1,'','toggle switching output and clipped surfaces'],
            ['Interactive','interactive','bool',1,'','toggle clipping with a widget or with a pre-defined scalar field'],
            ['ClipArrayName','array','str',1,'','name of the array with which to clip in case of non-interactive clipping'],
            ['ClipValue','value','float',1,'','scalar value at which to perform clipping in case of non-interactive clipping'],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter'],
            ['ClippedSurface','oclipped','vtkPolyData',1,'','the clipped surface','vmtksurfacewriter'],
            ['CutLines','ocutlines','vtkPolyData',1,'','the cutlines','vmtksurfacewriter'],
            ['Transform','otransform','vtkTransform',1,'','the output widget transform']
            ])

    def ClipCallback(self, obj):
        if self.ClipWidget.GetEnabled() != 1:
            return
        if self.WidgetType == "box":
            self.ClipWidget.GetPlanes(self.ClipFunction)
        elif self.WidgetType == "sphere":
            self.ClipWidget.GetSphere(self.ClipFunction)
        self.Clipper.Update()
        self.Surface.DeepCopy(self.Clipper.GetOutput())
        self.ClippedSurface.DeepCopy(self.Clipper.GetClippedOutput())
        self.Cutter.Update()
        self.CutLines.DeepCopy(self.Cutter.GetOutput())
        self.ClipWidget.Off()

    def InteractCallback(self, obj):
        pass
    #    if self.ClipWidget.GetEnabled() == 1:
    #        self.ClipWidget.SetEnabled(0)
    #    else:
    #        self.ClipWidget.SetEnabled(1)

    def Display(self):

        self.ClipWidget.SetInputData(self.Surface)
        self.ClipWidget.PlaceWidget()

        if self.Transform and self.WidgetType == "box":
            self.ClipWidget.SetTransform(self.Transform)
            self.ClipWidget.On()

        #self.vmtkRenderer.RenderWindowInteractor.Initialize()
        self.vmtkRenderer.Render()
        #self.vmtkRenderer.RenderWindowInteractor.Start()

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: no Surface.')

        self.Clipper = vtk.vtkClipPolyData()
        self.Clipper.SetInputData(self.Surface)
        self.Clipper.GenerateClippedOutputOn()
        self.Clipper.SetInsideOut(self.InsideOut)

        if self.Interactive:

            if self.WidgetType == "box":
                self.ClipFunction = vtk.vtkPlanes()
            elif self.WidgetType == "sphere":
                self.ClipFunction = vtk.vtkSphere()

            self.Clipper.SetClipFunction(self.ClipFunction)

            self.Cutter = vtk.vtkCutter()
            self.Cutter.SetInputData(self.Surface)
            self.Cutter.SetCutFunction(self.ClipFunction)

            self.ClippedSurface = vtk.vtkPolyData()
            self.CutLines = vtk.vtkPolyData()

            if not self.vmtkRenderer:
                self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
                self.vmtkRenderer.Initialize()
                self.OwnRenderer = 1

            self.vmtkRenderer.RegisterScript(self)

            mapper = vtk.vtkPolyDataMapper()
            mapper.SetInputData(self.Surface)
            mapper.ScalarVisibilityOff()
            self.Actor = vtk.vtkActor()
            self.Actor.SetMapper(mapper)
            self.vmtkRenderer.Renderer.AddActor(self.Actor)

            if self.WidgetType == "box":
                self.ClipWidget = vtk.vtkBoxWidget()
                self.ClipWidget.GetFaceProperty().SetColor(0.6,0.6,0.2)
                self.ClipWidget.GetFaceProperty().SetOpacity(0.25)
            elif self.WidgetType == "sphere":
                self.ClipWidget = vtk.vtkSphereWidget()
                self.ClipWidget.GetSphereProperty().SetColor(0.6,0.6,0.2)
                self.ClipWidget.GetSphereProperty().SetOpacity(0.25)
                self.ClipWidget.GetSelectedSphereProperty().SetColor(0.6,0.0,0.0)
                self.ClipWidget.GetSelectedSphereProperty().SetOpacity(0.75)
                self.ClipWidget.SetRepresentationToSurface()
                self.ClipWidget.SetPhiResolution(20)
                self.ClipWidget.SetThetaResolution(20)

            self.ClipWidget.SetInteractor(self.vmtkRenderer.RenderWindowInteractor)

            self.vmtkRenderer.AddKeyBinding('space','Clip.',self.ClipCallback)
            self.vmtkRenderer.AddKeyBinding('i','Interact.',self.InteractCallback)
            self.Display()

            self.Transform = vtk.vtkTransform()
            if self.WidgetType == "box":
                self.ClipWidget.GetTransform(self.Transform)

            if self.OwnRenderer:
                self.vmtkRenderer.Deallocate()

        else:

            self.Surface.GetPointData().SetActiveScalars(self.ClipArrayName)

            self.Clipper.GenerateClipScalarsOff()
            self.Clipper.SetValue(self.ClipValue)
            self.Clipper.Update()

            self.Cutter = vtk.vtkContourFilter()
            self.Cutter.SetInputData(self.Surface)
            self.Cutter.SetValue(0,self.ClipValue)
            self.Cutter.Update()

            self.Surface = self.Clipper.GetOutput()
            self.ClippedSurface = self.Clipper.GetClippedOutput()

            self.CutLines = self.Cutter.GetOutput()

        if self.CleanOutput == 1:

            cleaner = vtk.vtkCleanPolyData()
            cleaner.SetInputData(self.Surface)
            cleaner.Update()
            self.Surface = cleaner.GetOutput()

            cleaner = vtk.vtkCleanPolyData()
            cleaner.SetInputData(self.ClippedSurface)
            cleaner.Update()
            self.ClippedSurface = cleaner.GetOutput()

            cleaner = vtk.vtkCleanPolyData()
            cleaner.SetInputData(self.CutLines)
            cleaner.Update()
            stripper = vtk.vtkStripper()
            stripper.SetInputConnection(cleaner.GetOutputPort())
            stripper.Update()
            self.CutLines = stripper.GetOutput()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkbranchmetrics.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:48:31 $
## Version:   $Revision: 1.4 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vmtk import vtkvmtk
import sys

from vmtk import pypes


class vmtkBranchMetrics(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.Centerlines = None

        self.ComputeAbscissaMetric = 1
        self.ComputeAngularMetric = 1

        self.AbscissasArrayName = 'Abscissas'
        self.NormalsArrayName = 'ParallelTransportNormals'
        self.GroupIdsArrayName = 'GroupIds'
        self.CenterlineIdsArrayName = 'CenterlineIds'
        self.TractIdsArrayName = 'TractIds'
        self.RadiusArrayName = 'MaximumInscribedSphereRadius'
        self.BlankingArrayName = 'Blanking'

        self.AngularMetricArrayName = 'AngularMetric'
        self.AbscissaMetricArrayName = 'AbscissaMetric'

        self.SetScriptName('vmtkbranchmetrics')
        self.SetScriptDoc('Takes a centerline and input surface, already split into branches (with centerline attributes calculated) \
                           and calculates the surface angular and abscissa metric.')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','','vmtksurfacereader'],
            ['Centerlines','centerlines','vtkPolyData',1,'','','vmtksurfacereader'],
            ['ComputeAbscissaMetric','abscissametric','bool',1],
            ['ComputeAngularMetric','angularmetric','bool',1],
            ['AbscissasArrayName','abscissasarray','str',1],
            ['NormalsArrayName','normalsarray','str',1],
            ['GroupIdsArrayName','groupidsarray','str',1],
            ['CenterlineIdsArrayName','centerlineidsarray','str',1],
            ['TractIdsArrayName','tractidsarray','str',1],
            ['RadiusArrayName','radiusarray','str',1],
            ['BlankingArrayName','blankingarray','str',1],
            ['AngularMetricArrayName','angularmetricarray','str',1],
            ['AbscissaMetricArrayName','abscissametricarray','str',1]
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','','vmtksurfacewriter'],
            ['AngularMetricArrayName','angularmetricarray','str',1],
            ['AbscissaMetricArrayName','abscissametricarray','str',1]
            ])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        if self.Centerlines == None:
            self.PrintError('Error: No input centerlines.')

        if self.ComputeAngularMetric == 1:
            self.PrintLog('Computing angular metric')
            angularMetricFilter = vtkvmtk.vtkvmtkPolyDataCenterlineAngularMetricFilter()
            angularMetricFilter.SetInputData(self.Surface)
            angularMetricFilter.SetMetricArrayName(self.AngularMetricArrayName)
            angularMetricFilter.SetGroupIdsArrayName(self.GroupIdsArrayName)
            angularMetricFilter.SetCenterlines(self.Centerlines)
            angularMetricFilter.SetRadiusArrayName(self.RadiusArrayName)
            angularMetricFilter.SetCenterlineNormalsArrayName(self.NormalsArrayName)
            angularMetricFilter.SetCenterlineGroupIdsArrayName(self.GroupIdsArrayName)
            angularMetricFilter.SetCenterlineTractIdsArrayName(self.TractIdsArrayName)
            angularMetricFilter.UseRadiusInformationOff()
            angularMetricFilter.IncludeBifurcationsOff()
            angularMetricFilter.SetBlankingArrayName(self.BlankingArrayName)
            angularMetricFilter.SetCenterlineIdsArrayName(self.CenterlineIdsArrayName)
            angularMetricFilter.Update()
            self.Surface = angularMetricFilter.GetOutput()

        if self.ComputeAbscissaMetric == 1:
            self.PrintLog('Computing abscissa metric')
            abscissaMetricFilter = vtkvmtk.vtkvmtkPolyDataCenterlineAbscissaMetricFilter()
            abscissaMetricFilter.SetInputData(self.Surface)
            abscissaMetricFilter.SetMetricArrayName(self.AbscissaMetricArrayName)
            abscissaMetricFilter.SetGroupIdsArrayName(self.GroupIdsArrayName)
            abscissaMetricFilter.SetCenterlines(self.Centerlines)
            abscissaMetricFilter.SetRadiusArrayName(self.RadiusArrayName)
            abscissaMetricFilter.SetAbscissasArrayName(self.AbscissasArrayName)
            abscissaMetricFilter.SetCenterlineGroupIdsArrayName(self.GroupIdsArrayName)
            abscissaMetricFilter.SetCenterlineTractIdsArrayName(self.TractIdsArrayName)
            abscissaMetricFilter.UseRadiusInformationOff()
            abscissaMetricFilter.IncludeBifurcationsOn()
            abscissaMetricFilter.SetBlankingArrayName(self.BlankingArrayName)
            abscissaMetricFilter.SetCenterlineIdsArrayName(self.CenterlineIdsArrayName)
            abscissaMetricFilter.Update()
            self.Surface = abscissaMetricFilter.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkimagemorphology as morph


def test_dilate_grayscale_image(aorta_image, compare_images):
    name = __name__ + '_test_dilate_grayscale_image.mha'
    morpher = morph.vmtkImageMorphology()
    morpher.Image = aorta_image
    morpher.Operation = 'dilate'
    morpher.Execute()

    assert compare_images(morpher.Image, name) == True


def test_erode_grayscale_image(aorta_image, compare_images):
    name = __name__ + '_test_erode_grayscale_image.mha'
    morpher = morph.vmtkImageMorphology()
    morpher.Image = aorta_image
    morpher.Operation = 'erode'
    morpher.Execute()

    assert compare_images(morpher.Image, name) == True


def test_open_grayscale_image(aorta_image, compare_images):
    name = __name__ + '_test_open_grayscale_image.mha'
    morpher = morph.vmtkImageMorphology()
    morpher.Image = aorta_image
    morpher.Operation = 'open'
    morpher.Execute()

    assert compare_images(morpher.Image, name) == True


def test_close_grayscale_image(aorta_image, compare_images):
    name = __name__ + '_test_close_grayscale_image.mha'
    morpher = morph.vmtkImageMorphology()
    morpher.Image = aorta_image
    morpher.Operation = 'close'
    morpher.Execute()

    assert compare_images(morpher.Image, name) == True


def test_change_ball_radius_dilate_grayscale_image(aorta_image, compare_images):
    name = __name__ + '_test_change_ball_radius_dilate_grayscale_image.mha'
    morpher = morph.vmtkImageMorphology()
    morpher.Image = aorta_image
    morpher.Operation = 'dilate'
    morpher.BallRadius = [2, 2, 2]
    morpher.Execute()

    assert compare_images(morpher.Image, name) == True



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkcenterlinestonumpy.py,v $
## Language:  Python
## Date:      June 10, 2017
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this class was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo
##       The Jacobs Institute

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vtk.numpy_interface import dataset_adapter as dsa
import sys

from vmtk import vtkvmtk
from vmtk import vmtkrenderer
from vmtk import pypes

try:
    import numpy as np
except ImportError:
    raise ImportError('Unable to Import vmtkcenterlinestonumpy module, numpy is not installed')


# convenience class for nested dictionaries
class vividict(dict):
    def __missing__(self, key):
        value = self[key] = type(self)()
        return value


class vmtkCenterlinesToNumpy(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Centerlines = None
        self.ConvertCellToPoint = 0
        self.ArrayDict = vividict()

        self.SetScriptName('vmtkCenterlinesToNumpy')
        self.SetScriptDoc('Takes a VTK centerlines vtkPolyData file and returns a nested python dictionary containing numpy'
                          'arrays specifying vertex points, associated scalar data, and cell data yielding connectivity')
        self.SetInputMembers([
            ['Centerlines','i','vtkPolyData',1,'','the input centerlines','vmtksurfacereader'],
            ['ConvertCellToPoint','celltopoint','bool',1,'','convert cell data to point data']])
        self.SetOutputMembers([
            ['ArrayDict','o','dict',1,'','the output dictionary','vmtknumpywriter']])

    def Execute(self):

        if self.Centerlines == None:
            self.PrintError('Error: No input centerlines.')

        if self.ConvertCellToPoint == 1:
            self.PrintLog('converting cell data to point data')
            try:
                from vmtk import vmtksurfacecelldatatopointdata
            except ImportError:
                raise ImportError('unable to import vmtksurfacecelldata to point data module')

            centerlinesCellToPoint = vmtksurfacecelldatatopointdata.vmtkSurfaceCellDataToPointData()
            centerlinesCellToPoint.Surface = self.Centerlines
            centerlinesCellToPoint.Execute()
            self.PrintLog('wrapping vtkPolyData object')
            clWrapper = dsa.WrapDataObject(centerlinesCellToPoint.Surface)

        else:
            self.PrintLog('wrapping vtkPolyData object')
            clWrapper = dsa.WrapDataObject(self.Centerlines)

            self.PrintLog('converting cell data: ')
            for cellKey in clWrapper.CellData.keys():
                self.PrintLog(cellKey)
                self.ArrayDict['CellData'][cellKey] = np.array(clWrapper.CellData.GetArray(cellKey))

        self.PrintLog('converting points')
        self.ArrayDict['Points'] = np.array(clWrapper.Points)

        self.PrintLog('converting point data: ')
        for pointKey in clWrapper.PointData.keys():
            self.PrintLog(pointKey)
            self.ArrayDict['PointData'][pointKey] = np.array(clWrapper.PointData.GetArray(pointKey))

        self.PrintLog('converting cell connectivity list')
        cellPointIdsList = []
        numberOfCells = clWrapper.VTKObject.GetNumberOfCells()
        for cellId in range(numberOfCells):
            cell = clWrapper.VTKObject.GetCell(cellId)
            numberOfPointsPerCell = cell.GetNumberOfPoints()
            cellArray = np.zeros(shape=numberOfPointsPerCell, dtype=np.int32)
            for point in range(numberOfPointsPerCell):
                cellArray[point] = cell.GetPointId(point)
            cellPointIdsList.append(cellArray)

        self.ArrayDict['CellData']['CellPointIds'] = cellPointIdsList


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkimagecompose as comp


def test_multiply_images(aorta_image, compare_images):
    name = __name__ + '_test_multiply_images.mha'
    composer = comp.vmtkImageCompose()
    composer.Operation = 'multiply'
    composer.Image = aorta_image
    composer.Image2 = aorta_image
    composer.Execute()

    assert compare_images(composer.Image, name) == True


def test_subtract_images(aorta_image, compare_images):
    name = __name__ + '_test_subtract_images.mha'
    composer = comp.vmtkImageCompose()
    composer.Operation = 'subtract'
    composer.Image = aorta_image
    composer.Image2 = aorta_image
    composer.Execute()

    assert compare_images(composer.Image, name) == True


def test_negate_image2_and_multiply(aorta_image, compare_images):
    name = __name__ + '_test_negate_image2_and_multiply.mha'
    composer = comp.vmtkImageCompose()
    composer.Operation = 'multiply'
    composer.NegateImage2 = True
    composer.Image = aorta_image
    composer.Image2 = aorta_image
    composer.Execute()

    assert compare_images(composer.Image, name) == True


def test_negate_image2_and_min(aorta_image, compare_images):
    name = __name__ + '_test_negate_image2_and_min.mha'
    composer = comp.vmtkImageCompose()
    composer.Operation = 'min'
    composer.NegateImage2 = True
    composer.Image = aorta_image
    composer.Image2 = aorta_image
    composer.Execute()

    assert compare_images(composer.Image, name) == True


def test_negate_image2_and_max(aorta_image, compare_images):
    name = __name__ + '_test_negate_image2_and_max.mha'
    composer = comp.vmtkImageCompose()
    composer.Operation = 'max'
    composer.NegateImage2 = True
    composer.Image = aorta_image
    composer.Image2 = aorta_image
    composer.Execute()

    assert compare_images(composer.Image, name) == True



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkimageobjectenhancement as enhance


def test_enhance_image_with_defaults(aorta_image, compare_images):
    name = __name__ + '_test_enhance_image_with_defaults.mha'
    enhancer = enhance.vmtkImageObjectEnhancement()
    enhancer.Image = aorta_image
    enhancer.Execute()

    assert compare_images(enhancer.Image, name) == True


def test_enhance_image_with_scaled_objectiveness(aorta_image, compare_images):
    name = __name__ + '_test_enhance_image_with_scaled_objectiveness.mha'
    enhancer = enhance.vmtkImageObjectEnhancement()
    enhancer.Image = aorta_image
    enhancer.ScaledObjectness = True
    enhancer.Execute()

    assert compare_images(enhancer.Image, name) == True


@pytest.mark.parametrize("sigma_min,sigma_max,paramid", [
    (0.5, 1.0, '0'),
    (1.0, 2.0, '1'),
    (2.0, 1.0, '2'),
    (0.7, 2.3, '3')
])
def test_enhance_image_with_changing_sigma(aorta_image, compare_images, sigma_min, sigma_max, paramid):
    name = __name__ + '_test_enhance_image_with_changing_sigma_' + paramid + '.mha'
    enhancer = enhance.vmtkImageObjectEnhancement()
    enhancer.Image = aorta_image
    enhancer.SigmaMin = sigma_min
    enhancer.SigmaMax = sigma_max
    enhancer.Execute()

    assert compare_images(enhancer.Image, name) == True


@pytest.mark.parametrize("alpha,beta,gamma,paramid", [
    (0.5, 1.0, 5.0, '0'),
    (0.5, 0.5, 10.0, '1'),
    (2.0, 0.5, 10.0, '2'),
])
def test_enhance_image_with_changing_abg(aorta_image, compare_images, alpha, beta, gamma, paramid):
    name = __name__ + '_test_enhance_image_with_changing_abg_' + paramid + '.mha'
    enhancer = enhance.vmtkImageObjectEnhancement()
    enhancer.Image = aorta_image
    enhancer.Alpha = alpha
    enhancer.Beta = beta
    enhancer.Gamma = gamma
    enhancer.Execute()

    assert compare_images(enhancer.Image, name) == True


def test_enhance_image_three_sigma_steps(aorta_image, compare_images):
    name = __name__ + '_test_enhance_image_three_sigma_steps.mha'
    enhancer = enhance.vmtkImageObjectEnhancement()
    enhancer.Image = aorta_image
    enhancer.NumberOfSigmaSteps = 3
    enhancer.Execute()

    assert compare_images(enhancer.Image, name) == True
